@@ -1,6 +1,6 @@
 /*
    Copyright (c) 2006, 2013, Oracle and/or its affiliates.
-   Copyright (c) 2010, 2017, MariaDB
+   Copyright (c) 2010, 2016, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 #include "client_priv.h"
@@ -22,7 +22,7 @@
 
 #include <welcome_copyright_notice.h> /* ORACLE_WELCOME_COPYRIGHT_NOTICE */
 
-#define VER "1.3a"
+#define VER "1.4"
 
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
@@ -36,8 +36,8 @@
 # endif
 #endif
 
-static int phase = 1;
-static int phases_total = 4;
+static int phase = 0;
+static int phases_total = 6;
 static char mysql_path[FN_REFLEN];
 static char mysqlcheck_path[FN_REFLEN];
 
@@ -47,6 +47,8 @@ static my_bool opt_not_used, opt_silent;
 static uint my_end_arg= 0;
 static char *opt_user= (char*)"root";
 
+static my_bool upgrade_from_mysql;
+
 static DYNAMIC_STRING ds_args;
 static DYNAMIC_STRING conn_args;
 
@@ -145,22 +147,22 @@ static struct my_option my_long_options[]=
 #include <sslopt-longopts.h>
   {"tmpdir", 't', "Directory for temporary files.",
    0, 0, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-  {"upgrade-system-tables", 's', "Only upgrade the system tables "
-   "do not try to upgrade the data.",
+  {"upgrade-system-tables", 's', "Only upgrade the system tables in the mysql database. Tables in other databases are not checked or touched.",
    &opt_systables_only, &opt_systables_only, 0,
    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
 #define USER_OPT (array_elements(my_long_options) - 6)
   {"user", 'u', "User for login if not current user.", &opt_user,
    &opt_user, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-  {"verbose", 'v', "Display more output about the process.",
+  {"verbose", 'v', "Display more output about the process; Using it twice will print connection argument; Using it 3 times will print out all CHECK, RENAME and ALTER TABLE during the check phase.",
    &opt_not_used, &opt_not_used, 0, GET_BOOL, NO_ARG, 1, 0, 0, 0, 0, 0},
   {"version", 'V', "Output version information and exit.", 0, 0, 0,
    GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
-  {"version-check", 'k', "Run this program only if its \'server version\' "
-   "matches the version of the server to which it's connecting, (enabled by "
-   "default); use --skip-version-check to avoid this check. Note: the \'server "
-   "version\' of the program is the version of the MySQL server with which it "
-   "was built/distributed.", &opt_version_check, &opt_version_check, 0,
+  {"version-check", 'k',
+   "Run this program only if its \'server version\' "
+   "matches the version of the server to which it's connecting. "
+   "Note: the \'server version\' of the program is the version of the MariaDB "
+   "server with which it was built/distributed.",
+   &opt_version_check, &opt_version_check, 0,
    GET_BOOL, NO_ARG, 1, 0, 0, 0, 0, 0},
   {"write-binlog", OPT_WRITE_BINLOG, "All commands including those, "
    "issued by mysqlcheck, are written to the binary log.",
@@ -381,6 +383,9 @@ static int run_command(char* cmd,
   FILE *res_file;
   int error;
 
+  if (opt_verbose >= 4)
+    puts(cmd);
+
   if (!(res_file= popen(cmd, "r")))
     die("popen(\"%s\", \"r\") failed", cmd);
 
@@ -605,7 +610,7 @@ static int extract_variable_from_show(DYNAMIC_STRING* ds, char* value)
   if ((value_end= strchr(value_start, '\n')) == NULL)
     return 1; /* Unexpected result */
 
-  len= (size_t) min(FN_REFLEN, value_end-value_start);
+  len= (size_t) MY_MIN(FN_REFLEN, value_end-value_start);
   strncpy(value, value_start, len);
   value[len]= '\0';
   return 0;
@@ -639,7 +644,7 @@ static int get_upgrade_info_file_name(char* name)
 /*
   Read the content of mysql_upgrade_info file and
   compare the version number form file against
-  version number wich mysql_upgrade was compiled for
+  version number which mysql_upgrade was compiled for
 
   NOTE
   This is an optimization to avoid running mysql_upgrade
@@ -743,26 +748,31 @@ static void print_conn_args(const char *tool_name)
   in the server using "mysqlcheck --check-upgrade .."
 */
 
-static int run_mysqlcheck_upgrade(void)
+static int run_mysqlcheck_upgrade(my_bool mysql_db_only)
 {
+  const char *what= mysql_db_only ? "mysql database" : "tables";
+  const char *arg1= mysql_db_only ? "--databases" : "--all-databases";
+  const char *arg2= mysql_db_only ? "mysql" : "--skip-database=mysql";
   int retch;
-  if (opt_systables_only)
+  if (opt_systables_only && !mysql_db_only)
   {
-    verbose("Phase %d/%d: Checking and upgrading tables... Skipped",
-            phase++, phases_total);
+    verbose("Phase %d/%d: Checking and upgrading %s... Skipped",
+            ++phase, phases_total, what);
     return 0;
   }
-  verbose("Phase %d/%d: Checking and upgrading tables", phase++, phases_total);
+  verbose("Phase %d/%d: Checking and upgrading %s", ++phase, phases_total, what);
   print_conn_args("mysqlcheck");
   retch= run_tool(mysqlcheck_path,
                   NULL, /* Send output from mysqlcheck directly to screen */
                   defaults_file,
                   "--check-upgrade",
-                  "--all-databases",
                   "--auto-repair",
-                  !opt_silent || opt_verbose ? "--verbose": "",
+                  !opt_silent || opt_verbose >= 1 ? "--verbose" : "",
+                  opt_verbose >= 2 ? "--verbose" : "",
+                  opt_verbose >= 3 ? "--verbose" : "",
                   opt_silent ? "--silent": "",
                   opt_write_binlog ? "--write-binlog" : "--skip-write-binlog",
+                  arg1, arg2,
                   "2>&1",
                   NULL);
   return retch;
@@ -793,18 +803,23 @@ static my_bool is_mysql()
 static int run_mysqlcheck_views(void)
 {
   const char *upgrade_views="--process-views=YES";
-  if (is_mysql())
+  if (upgrade_from_mysql)
   {
+    /*
+      this has to ignore opt_systables_only, because upgrade_from_mysql
+      is determined by analyzing systables. if we honor opt_systables_only
+      here, views won't be fixed by subsequent mysql_upgrade runs
+    */
     upgrade_views="--process-views=UPGRADE_FROM_MYSQL";
-    verbose("Phase %d/%d: Fixing views from mysql", phase++, phases_total);
+    verbose("Phase %d/%d: Fixing views from mysql", ++phase, phases_total);
   }
   else if (opt_systables_only)
   {
-    verbose("Phase %d/%d: Fixing views... Skipped", phase++, phases_total);
+    verbose("Phase %d/%d: Fixing views... Skipped", ++phase, phases_total);
     return 0;
   }
   else
-    verbose("Phase %d/%d: Fixing views", phase++, phases_total);
+    verbose("Phase %d/%d: Fixing views", ++phase, phases_total);
 
   print_conn_args("mysqlcheck");
   return run_tool(mysqlcheck_path,
@@ -825,11 +840,11 @@ static int run_mysqlcheck_fixnames(void)
   if (opt_systables_only)
   {
     verbose("Phase %d/%d: Fixing table and database names ... Skipped",
-            phase++, phases_total);
+            ++phase, phases_total);
     return 0;
   }
   verbose("Phase %d/%d: Fixing table and database names",
-          phase++, phases_total);
+          ++phase, phases_total);
   print_conn_args("mysqlcheck");
   return run_tool(mysqlcheck_path,
                   NULL, /* Send output from mysqlcheck directly to screen */
@@ -837,7 +852,9 @@ static int run_mysqlcheck_fixnames(void)
                   "--all-databases",
                   "--fix-db-names",
                   "--fix-table-names",
-                  opt_verbose ? "--verbose": "",
+                  opt_verbose >= 1 ? "--verbose" : "",
+                  opt_verbose >= 2 ? "--verbose" : "",
+                  opt_verbose >= 3 ? "--verbose" : "",
                   opt_silent ? "--silent": "",
                   opt_write_binlog ? "--write-binlog" : "--skip-write-binlog",
                   "2>&1",
@@ -850,6 +867,7 @@ static const char *expected_errors[]=
   "ERROR 1060", /* Duplicate column name */
   "ERROR 1061", /* Duplicate key name */
   "ERROR 1054", /* Unknown column */
+  "ERROR 1290", /* RR_OPTION_PREVENTS_STATEMENT */
   0
 };
 
@@ -903,15 +921,35 @@ static void print_line(char* line)
 static int run_sql_fix_privilege_tables(void)
 {
   int found_real_errors= 0;
+  const char **query_ptr;
+  DYNAMIC_STRING ds_script;
   DYNAMIC_STRING ds_result;
   DBUG_ENTER("run_sql_fix_privilege_tables");
 
+  if (init_dynamic_string(&ds_script, "", 65536, 1024))
+    die("Out of memory");
+
   if (init_dynamic_string(&ds_result, "", 512, 512))
     die("Out of memory");
 
   verbose("Phase %d/%d: Running 'mysql_fix_privilege_tables'",
-          phase++, phases_total);
-  run_query(mysql_fix_privilege_tables,
+          ++phase, phases_total);
+
+  /*
+    Individual queries can not be executed independently by invoking
+    a forked mysql client, because the script uses session variables
+    and prepared statements.
+  */
+  for ( query_ptr= &mysql_fix_privilege_tables[0];
+        *query_ptr != NULL;
+        query_ptr++
+      )
+  {
+    if (strcasecmp(*query_ptr, "flush privileges;\n"))
+      dynstr_append(&ds_script, *query_ptr);
+  }
+
+  run_query(ds_script.str,
             &ds_result, /* Collect result */
             TRUE);
 
@@ -939,6 +977,7 @@ static int run_sql_fix_privilege_tables(void)
   }
 
   dynstr_free(&ds_result);
+  dynstr_free(&ds_script);
   DBUG_RETURN(found_real_errors);
 }
 
@@ -1073,25 +1112,32 @@ int main(int argc, char **argv)
   if (opt_version_check && check_version_match())
     die("Upgrade failed");
 
+  upgrade_from_mysql= is_mysql();
+
   /*
     Run "mysqlcheck" and "mysql_fix_privilege_tables.sql"
   */
-  if (run_mysqlcheck_views() || run_mysqlcheck_fixnames() ||
-      run_mysqlcheck_upgrade() || run_sql_fix_privilege_tables())
-  {
-    /*
-      The upgrade failed to complete in some way or another,
-      significant error message should have been printed to the screen
-    */
+  if (run_mysqlcheck_upgrade(TRUE) ||
+      run_mysqlcheck_views() ||
+      run_sql_fix_privilege_tables() ||
+      run_mysqlcheck_fixnames() ||
+      run_mysqlcheck_upgrade(FALSE))
     die("Upgrade failed" );
-  }
+
+  verbose("Phase %d/%d: Running 'FLUSH PRIVILEGES'", ++phase, phases_total);
+  if (run_query("FLUSH PRIVILEGES", NULL, TRUE))
+    die("Upgrade failed" );
+
   verbose("OK");
 
   /* Create a file indicating upgrade has been performed */
   create_mysql_upgrade_info_file();
 
+  DBUG_ASSERT(phase == phases_total);
+
 end:
   free_used_memory();
   my_end(my_end_arg);
   exit(0);
 }
+
