@@ -1,6 +1,6 @@
 /*
-   Copyright (c) 2001, 2011, Oracle and/or its affiliates.
-   Copyright (c) 2010, 2017, MariaDB
+   Copyright (c) 2001, 2013, Oracle and/or its affiliates.
+   Copyright (c) 2010, 2016, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,12 +13,12 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 /* By Jani Tolonen, 2001-04-20, MySQL Development Team */
 
-#define CHECK_VERSION "2.7.2-MariaDB"
+#define CHECK_VERSION "2.7.4-MariaDB"
 
 #include "client_priv.h"
 #include <m_ctype.h>
@@ -43,7 +43,7 @@ static my_bool opt_alldbs = 0, opt_check_only_changed = 0, opt_extended = 0,
                opt_silent = 0, opt_auto_repair = 0, ignore_errors = 0,
                tty_password= 0, opt_frm= 0, debug_info_flag= 0, debug_check_flag= 0,
                opt_fix_table_names= 0, opt_fix_db_names= 0, opt_upgrade= 0,
-               opt_do_tables= 1;
+               opt_persistent_all= 0, opt_do_tables= 1;
 static my_bool opt_write_binlog= 1, opt_flush_tables= 0;
 static uint verbose = 0, opt_mysql_port=0;
 static int my_end_arg;
@@ -52,6 +52,7 @@ static char *opt_password = 0, *current_user = 0,
 	    *default_charset= 0, *current_host= 0;
 static char *opt_plugin_dir= 0, *opt_default_auth= 0;
 static int first_error = 0;
+static char *opt_skip_database;
 DYNAMIC_ARRAY tables4repair, tables4rebuild, alter_table_cmds;
 DYNAMIC_ARRAY views4repair;
 static char *shared_memory_base_name=0;
@@ -159,6 +160,10 @@ static struct my_option my_long_options[] =
   {"password", 'p',
    "Password to use when connecting to server. If password is not given, it's solicited on the tty.",
    0, 0, 0, GET_STR, OPT_ARG, 0, 0, 0, 0, 0, 0},
+  {"persistent", 'Z',
+   "When using ANALYZE TABLE use the PERSISTENT FOR ALL option.",
+   &opt_persistent_all, &opt_persistent_all, 0, GET_BOOL, NO_ARG,
+   0, 0, 0, 0, 0, 0},
 #ifdef __WIN__
   {"pipe", 'W', "Use named pipes to connect to server.", 0, 0, 0, GET_NO_ARG,
    NO_ARG, 0, 0, 0, 0, 0, 0},
@@ -190,6 +195,9 @@ static struct my_option my_long_options[] =
 #endif
   {"silent", 's', "Print only error messages.", &opt_silent,
    &opt_silent, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
+  {"skip_database", 0, "Don't process the database specified as argument", 
+   &opt_skip_database, &opt_skip_database, 0, GET_STR, REQUIRED_ARG, 
+   0, 0, 0, 0, 0, 0},
   {"socket", 'S', "The socket file to use for connection.",
    &opt_mysql_unix_port, &opt_mysql_unix_port, 0, GET_STR,
    REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
@@ -204,8 +212,8 @@ static struct my_option my_long_options[] =
   {"user", 'u', "User for login if not current user.", &current_user,
    &current_user, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 #endif
-  {"verbose", 'v', "Print info about the various stages.", 0, 0, 0, GET_NO_ARG,
-   NO_ARG, 0, 0, 0, 0, 0, 0},
+  {"verbose", 'v', "Print info about the various stages; Using it 3 times will print out all CHECK, RENAME and ALTER TABLE during the check phase.",
+   0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"version", 'V', "Output version information and exit.", 0, 0, 0, GET_NO_ARG,
    NO_ARG, 0, 0, 0, 0, 0, 0},
   {"process-views", 0,
@@ -232,7 +240,7 @@ static int process_selected_tables(char *db, char **table_names, int tables);
 static int process_all_tables_in_db(char *database);
 static int process_one_db(char *database);
 static int use_db(char *database);
-static int handle_request_for_tables(char *, size_t, my_bool, my_bool);
+static int handle_request_for_tables(char *tables, size_t length, my_bool view);
 static int dbConnect(char *host, char *user,char *passwd);
 static void dbDisconnect(char *host);
 static void DBerror(MYSQL *mysql, const char *when);
@@ -266,8 +274,11 @@ static void usage(void)
   puts("mysqlrepair:   The default option will be -r");
   puts("mysqlanalyze:  The default option will be -a");
   puts("mysqloptimize: The default option will be -o\n");
-  puts("Please consult the MariaDB/MySQL knowledgebase at");
-  puts("http://kb.askmonty.org/v/mysqlcheck for latest information about");
+  printf("Usage: %s [OPTIONS] database [tables]\n", my_progname);
+  printf("OR     %s [OPTIONS] --databases DB1 [DB2 DB3...]\n",
+	 my_progname);
+  puts("Please consult the MariaDB Knowledge Base at");
+  puts("https://mariadb.com/kb/en/mysqlcheck for latest information about");
   puts("this program.");
   print_defaults("my", load_default_groups);
   puts("");
@@ -566,7 +577,7 @@ static int process_selected_tables(char *db, char **table_names, int tables)
     }
     *--end = 0;
     handle_request_for_tables(table_names_comma_sep + 1, tot_length - 1,
-                              opt_do_views != 0, opt_all_in_1);
+                              opt_do_views != 0);
     my_free(table_names_comma_sep);
   }
   else
@@ -577,7 +588,7 @@ static int process_selected_tables(char *db, char **table_names, int tables)
       view= is_view(table);
       if (view < 0)
         continue;
-      handle_request_for_tables(table, table_len, view == 1, opt_all_in_1);
+      handle_request_for_tables(table, table_len, (view == 1));
     }
   DBUG_RETURN(0);
 } /* process_selected_tables */
@@ -605,9 +616,13 @@ static char *fix_table_name(char *dest, char *src)
   *dest++= '`';
   for (; *src; src++)
   {
-    if (*src == '`')
+    switch (*src) {
+    case '`':            /* escape backtick character */
       *dest++= '`';
-    *dest++= *src;
+      /* fall through */
+    default:
+      *dest++= *src;
+    }
   }
   *dest++= '`';
 
@@ -696,9 +711,9 @@ static int process_all_tables_in_db(char *database)
     *--end = 0;
     *--views_end = 0;
     if (tot_length)
-      handle_request_for_tables(tables + 1, tot_length - 1, FALSE, opt_all_in_1);
+      handle_request_for_tables(tables + 1, tot_length - 1, FALSE);
     if (tot_views_length)
-      handle_request_for_tables(views + 1, tot_views_length - 1, TRUE, opt_all_in_1);
+      handle_request_for_tables(views + 1, tot_views_length - 1, TRUE);
     my_free(tables);
     my_free(views);
   }
@@ -724,7 +739,7 @@ static int process_all_tables_in_db(char *database)
            !strcmp(row[0], "slow_log")))
         continue;                               /* Skip logging tables */
 
-      handle_request_for_tables(row[0], fixed_name_length(row[0]), view, opt_all_in_1);
+      handle_request_for_tables(row[0], fixed_name_length(row[0]), view);
     }
   }
   mysql_free_result(res);
@@ -732,8 +747,10 @@ static int process_all_tables_in_db(char *database)
 } /* process_all_tables_in_db */
 
 
-static int run_query(const char *query)
+static int run_query(const char *query, my_bool log_query)
 {
+  if (verbose >=3 && log_query)
+    puts(query);
   if (mysql_query(sock, query))
   {
     fprintf(stderr, "Failed to %s\n", query);
@@ -755,7 +772,7 @@ static int fix_table_storage_name(const char *name)
   my_snprintf(qbuf, sizeof(qbuf), "RENAME TABLE %`s TO %`s",
               name, name + 9);
 
-  rc= run_query(qbuf);
+  rc= run_query(qbuf, 1);
   if (verbose)
     printf("%-50s %s\n", name, rc ? "FAILED" : "OK");
   DBUG_RETURN(rc);
@@ -771,7 +788,7 @@ static int fix_database_storage_name(const char *name)
     DBUG_RETURN(1);
   my_snprintf(qbuf, sizeof(qbuf), "ALTER DATABASE %`s UPGRADE DATA DIRECTORY "
               "NAME", name);
-  rc= run_query(qbuf);
+  rc= run_query(qbuf, 1);
   if (verbose)
     printf("%-50s %s\n", name, rc ? "FAILED" : "OK");
   DBUG_RETURN(rc);
@@ -783,17 +800,23 @@ static int rebuild_table(char *name)
   int rc= 0;
   DBUG_ENTER("rebuild_table");
 
-  query= (char*)my_malloc(sizeof(char) * (12 + strlen(name) + 6 + 1),
+  query= (char*)my_malloc(sizeof(char) * (12 + fixed_name_length(name) + 6 + 1),
                           MYF(MY_WME));
   if (!query)
     DBUG_RETURN(1);
-  ptr= strxmov(query, "ALTER TABLE ", name, " FORCE", NullS);
+  ptr= strmov(query, "ALTER TABLE ");
+  ptr= fix_table_name(ptr, name);
+  ptr= strxmov(ptr, " FORCE", NullS);
+  if (verbose >= 3)
+    puts(query);
   if (mysql_real_query(sock, query, (ulong)(ptr - query)))
   {
     fprintf(stderr, "Failed to %s\n", query);
     fprintf(stderr, "Error: %s\n", mysql_error(sock));
     rc= 1;
   }
+  if (verbose)
+    printf("%-50s %s\n", name, rc ? "FAILED" : "FIXED");
   my_free(query);
   DBUG_RETURN(rc);
 }
@@ -802,6 +825,9 @@ static int process_one_db(char *database)
 {
   DBUG_ENTER("process_one_db");
 
+  if (opt_skip_database && !strcmp(database, opt_skip_database))
+    DBUG_RETURN(0);
+
   if (verbose)
     puts(database);
   if (what_to_do == DO_FIX_NAMES)
@@ -840,11 +866,10 @@ static int use_db(char *database)
 static int disable_binlog()
 {
   const char *stmt= "SET SQL_LOG_BIN=0";
-  return run_query(stmt);
+  return run_query(stmt, 0);
 }
 
-static int handle_request_for_tables(char *tables, size_t length,
-                                     my_bool view, my_bool dont_quote)
+static int handle_request_for_tables(char *tables, size_t length, my_bool view)
 {
   char *query, *end, options[100], message[100];
   char table_name_buff[NAME_CHAR_LEN*2*2+1], *table_name;
@@ -890,6 +915,7 @@ static int handle_request_for_tables(char *tables, size_t length,
   case DO_ANALYZE:
     DBUG_ASSERT(!view);
     op= (opt_write_binlog) ? "ANALYZE" : "ANALYZE NO_WRITE_TO_BINLOG";
+    if (opt_persistent_all) end = strmov(end, " PERSISTENT FOR ALL");
     break;
   case DO_OPTIMIZE:
     DBUG_ASSERT(!view);
@@ -902,7 +928,7 @@ static int handle_request_for_tables(char *tables, size_t length,
 
   if (!(query =(char *) my_malloc(query_size, MYF(MY_WME))))
     DBUG_RETURN(1);
-  if (dont_quote)
+  if (opt_all_in_1)
   {
     DBUG_ASSERT(strlen(op)+strlen(tables)+strlen(options)+8+1 <= query_size);
 
@@ -917,12 +943,14 @@ static int handle_request_for_tables(char *tables, size_t length,
 
     org= ptr= strmov(strmov(query, op), tab_view);
     ptr= fix_table_name(ptr, tables);
-    strmake(table_name_buff, org, min((int) sizeof(table_name_buff)-1,
-                                      (int) (ptr - org)));
+    strmake(table_name_buff, org, MY_MIN((int) sizeof(table_name_buff)-1,
+                                         (int) (ptr - org)));
     table_name= table_name_buff;
     ptr= strxmov(ptr, " ", options, NullS);
     query_length= (size_t) (ptr - query);
   }
+  if (verbose >= 3)
+    puts(query);
   if (mysql_real_query(sock, query, query_length))
   {
     sprintf(message, "when executing '%s%s... %s'", op, tab_view, options);
@@ -945,13 +973,6 @@ static int handle_request_for_tables(char *tables, size_t length,
   DBUG_RETURN(0);
 }
 
-static void insert_table_name(DYNAMIC_ARRAY *arr, char *in, size_t dblen)
-{
-  char buf[NAME_LEN*2+2];
-  in[dblen]= 0;
-  my_snprintf(buf, sizeof(buf), "%`s.%`s", in, in + dblen + 1);
-  insert_dynamic(arr, (uchar*) buf);
-}
 
 static void print_result()
 {
@@ -959,13 +980,16 @@ static void print_result()
   MYSQL_ROW row;
   char prev[(NAME_LEN+9)*3+2];
   char prev_alter[MAX_ALTER_STR_SIZE];
-  size_t length_of_db= strlen(sock->db);
+  char *db_name;
+  uint length_of_db;
   uint i;
   my_bool found_error=0, table_rebuild=0;
   DYNAMIC_ARRAY *array4repair= &tables4repair;
   DBUG_ENTER("print_result");
 
   res = mysql_use_result(sock);
+  db_name= sock->db;
+  length_of_db= strlen(db_name);
 
   prev[0] = '\0';
   prev_alter[0]= 0;
@@ -989,10 +1013,16 @@ static void print_result()
           if (prev_alter[0])
             insert_dynamic(&alter_table_cmds, (uchar*) prev_alter);
           else
-            insert_table_name(&tables4rebuild, prev, length_of_db);
+          {
+            char *table_name= prev + (length_of_db+1);
+            insert_dynamic(&tables4rebuild, (uchar*) table_name);
+          }
         }
         else
-          insert_table_name(array4repair, prev, length_of_db);
+        {
+          char *table_name= prev + (length_of_db+1);
+          insert_dynamic(array4repair, table_name);
+        }
       }
       array4repair= &tables4repair;
       found_error=0;
@@ -1019,31 +1049,9 @@ static void print_result()
         printf("%s\n%-9s: %s", row[0], row[2], row[3]);
       if (opt_auto_repair && strcmp(row[2],"note"))
       {
-        const char *alter_txt= strstr(row[3], "ALTER TABLE");
         found_error=1;
-        if (alter_txt)
-        {
+        if (opt_auto_repair && strstr(row[3], "ALTER TABLE") != NULL)
           table_rebuild=1;
-          if (!strncmp(row[3], KEY_PARTITIONING_CHANGED_STR,
-                       strlen(KEY_PARTITIONING_CHANGED_STR)) &&
-              strstr(alter_txt, "PARTITION BY"))
-          {
-            if (strlen(alter_txt) >= MAX_ALTER_STR_SIZE)
-            {
-              printf("Error: Alter command too long (>= %d),"
-                     " please do \"%s\" or dump/reload to fix it!\n",
-                     MAX_ALTER_STR_SIZE,
-                     alter_txt);
-              table_rebuild= 0;
-              prev_alter[0]= 0;
-            }
-            else
-            {
-              strncpy(prev_alter, alter_txt, MAX_ALTER_STR_SIZE-1);
-              prev_alter[MAX_ALTER_STR_SIZE-1]= 0;
-            }
-          }
-        }
       }
     }
     else
@@ -1057,12 +1065,18 @@ static void print_result()
     if (table_rebuild)
     {
       if (prev_alter[0])
-        insert_dynamic(&alter_table_cmds, (uchar*) prev_alter);
+        insert_dynamic(&alter_table_cmds, prev_alter);
       else
-        insert_table_name(&tables4rebuild, prev, length_of_db);
+      {
+        char *table_name= prev + (length_of_db+1);
+        insert_dynamic(&tables4rebuild, table_name);
+      }
     }
     else
-      insert_table_name(array4repair, prev, length_of_db);
+    {
+      char *table_name= prev + (length_of_db+1);
+      insert_dynamic(array4repair, table_name);
+    }
   }
   mysql_free_result(res);
   DBUG_VOID_RETURN;
@@ -1081,8 +1095,12 @@ static int dbConnect(char *host, char *user, char *passwd)
     mysql_options(&mysql_connection, MYSQL_OPT_COMPRESS, NullS);
 #ifdef HAVE_OPENSSL
   if (opt_use_ssl)
+  {
     mysql_ssl_set(&mysql_connection, opt_ssl_key, opt_ssl_cert, opt_ssl_ca,
 		  opt_ssl_capath, opt_ssl_cipher);
+    mysql_options(&mysql_connection, MYSQL_OPT_SSL_CRL, opt_ssl_crl);
+    mysql_options(&mysql_connection, MYSQL_OPT_SSL_CRLPATH, opt_ssl_crlpath);
+  }
 #endif
   if (opt_protocol)
     mysql_options(&mysql_connection,MYSQL_OPT_PROTOCOL,(char*)&opt_protocol);
@@ -1096,6 +1114,9 @@ static int dbConnect(char *host, char *user, char *passwd)
     mysql_options(&mysql_connection, MYSQL_DEFAULT_AUTH, opt_default_auth);
 
   mysql_options(&mysql_connection, MYSQL_SET_CHARSET_NAME, default_charset);
+  mysql_options(&mysql_connection, MYSQL_OPT_CONNECT_ATTR_RESET, 0);
+  mysql_options4(&mysql_connection, MYSQL_OPT_CONNECT_ATTR_ADD,
+                 "program_name", "mysqlcheck");
   if (!(sock = mysql_real_connect(&mysql_connection, host, user, passwd,
          NULL, opt_mysql_port, opt_mysql_unix_port, 0)))
   {
@@ -1172,10 +1193,14 @@ int main(int argc, char **argv)
   }
 
   if (opt_auto_repair &&
-      (my_init_dynamic_array(&tables4repair, sizeof(char)*(NAME_LEN*2+2),16,64) ||
-       my_init_dynamic_array(&views4repair, sizeof(char)*(NAME_LEN*2+2),16,64) ||
-       my_init_dynamic_array(&tables4rebuild, sizeof(char)*(NAME_LEN*2+2),16,64) ||
-       my_init_dynamic_array(&alter_table_cmds, MAX_ALTER_STR_SIZE, 0, 1)))
+      (my_init_dynamic_array(&tables4repair, sizeof(char)*(NAME_LEN*2+2),16,
+                             64, MYF(0)) ||
+       my_init_dynamic_array(&views4repair, sizeof(char)*(NAME_LEN*2+2),16,
+                             64, MYF(0)) ||
+       my_init_dynamic_array(&tables4rebuild, sizeof(char)*(NAME_LEN*2+2),16,
+                             64, MYF(0)) ||
+       my_init_dynamic_array(&alter_table_cmds, MAX_ALTER_STR_SIZE, 0, 1,
+                             MYF(0))))
     goto end;
 
   if (opt_alldbs)
@@ -1196,21 +1221,21 @@ int main(int argc, char **argv)
     for (i = 0; i < tables4repair.elements ; i++)
     {
       char *name= (char*) dynamic_array_ptr(&tables4repair, i);
-      handle_request_for_tables(name, fixed_name_length(name), FALSE, TRUE);
+      handle_request_for_tables(name, fixed_name_length(name), FALSE);
     }
     for (i = 0; i < tables4rebuild.elements ; i++)
       rebuild_table((char*) dynamic_array_ptr(&tables4rebuild, i));
     for (i = 0; i < alter_table_cmds.elements ; i++)
-      run_query((char*) dynamic_array_ptr(&alter_table_cmds, i));
+      run_query((char*) dynamic_array_ptr(&alter_table_cmds, i), 1);
     if (!opt_silent && views4repair.elements)
       puts("\nRepairing views");
     for (i = 0; i < views4repair.elements ; i++)
     {
       char *name= (char*) dynamic_array_ptr(&views4repair, i);
-      handle_request_for_tables(name, fixed_name_length(name), TRUE, TRUE);
+      handle_request_for_tables(name, fixed_name_length(name), TRUE);
     }
   }
-  ret= test(first_error);
+  ret= MY_TEST(first_error);
 
  end:
   dbDisconnect(current_host);
