@@ -1,6 +1,6 @@
 /*
    Copyright (c) 2000, 2015, Oracle and/or its affiliates.
-   Copyright (c) 2011, 2017, MariaDB
+   Copyright (c) 2011, 2016, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 /*
@@ -37,7 +37,6 @@
 
 /* Global Thread counter */
 uint counter= 0;
-pthread_mutex_t init_mutex;
 pthread_mutex_t counter_mutex;
 pthread_cond_t count_threshhold;
 
@@ -422,19 +421,8 @@ static MYSQL *db_connect(char *host, char *database,
   MYSQL *mysql;
   if (verbose)
     fprintf(stdout, "Connecting to %s\n", host ? host : "localhost");
-  if (opt_use_threads && !lock_tables)
-  {
-    pthread_mutex_lock(&init_mutex);
-    if (!(mysql= mysql_init(NULL)))
-    {
-      pthread_mutex_unlock(&init_mutex);
-      return 0;
-    }
-    pthread_mutex_unlock(&init_mutex);
-  }
-  else
-    if (!(mysql= mysql_init(NULL)))
-      return 0;
+  if (!(mysql= mysql_init(NULL)))
+    return 0;
   if (opt_compress)
     mysql_options(mysql,MYSQL_OPT_COMPRESS,NullS);
   if (opt_local_file)
@@ -442,8 +430,12 @@ static MYSQL *db_connect(char *host, char *database,
 		  (char*) &opt_local_file);
 #ifdef HAVE_OPENSSL
   if (opt_use_ssl)
+  {
     mysql_ssl_set(mysql, opt_ssl_key, opt_ssl_cert, opt_ssl_ca,
 		  opt_ssl_capath, opt_ssl_cipher);
+    mysql_options(mysql, MYSQL_OPT_SSL_CRL, opt_ssl_crl);
+    mysql_options(mysql, MYSQL_OPT_SSL_CRLPATH, opt_ssl_crlpath);
+  }
   mysql_options(mysql,MYSQL_OPT_SSL_VERIFY_SERVER_CERT,
                 (char*)&opt_ssl_verify_server_cert);
 #endif
@@ -461,6 +453,9 @@ static MYSQL *db_connect(char *host, char *database,
     mysql_options(mysql, MYSQL_DEFAULT_AUTH, opt_default_auth);
 
   mysql_options(mysql, MYSQL_SET_CHARSET_NAME, default_charset);
+  mysql_options(mysql, MYSQL_OPT_CONNECT_ATTR_RESET, 0);
+  mysql_options4(mysql, MYSQL_OPT_CONNECT_ATTR_ADD,
+                 "program_name", "mysqlimport");
   if (!(mysql_real_connect(mysql,host,user,passwd,
                            database,opt_mysql_port,opt_mysql_unix_port,
                            0)))
@@ -489,7 +484,6 @@ static void db_disconnect(char *host, MYSQL *mysql)
 }
 
 
-
 static void safe_exit(int error, MYSQL *mysql)
 {
   if (error && ignore_errors)
@@ -508,7 +502,10 @@ static void safe_exit(int error, MYSQL *mysql)
   free_defaults(argv_to_free);
   mysql_library_end();
   my_free(opt_password);
-  my_end(my_end_arg);
+  if (error)
+    sf_leaking_memory= 1; /* dirty exit, some threads are still running */
+  else
+    my_end(my_end_arg); /* clean exit */
   exit(error);
 }
 
@@ -617,7 +614,7 @@ error:
   pthread_cond_signal(&count_threshhold);
   pthread_mutex_unlock(&counter_mutex);
   mysql_thread_end();
-  pthread_exit(0);
+
   return 0;
 }
 
@@ -641,31 +638,15 @@ int main(int argc, char **argv)
 
   if (opt_use_threads && !lock_tables)
   {
-    char **save_argv;
-    uint worker_thread_count= 0, table_count= 0, i= 0;
-    pthread_t *worker_threads;       /* Thread descriptor */
-    pthread_attr_t attr;             /* Thread attributes */
+    pthread_t mainthread;            /* Thread descriptor */
+    pthread_attr_t attr;          /* Thread attributes */
     pthread_attr_init(&attr);
     pthread_attr_setdetachstate(&attr,
-                                PTHREAD_CREATE_JOINABLE);
+                                PTHREAD_CREATE_DETACHED);
 
-    pthread_mutex_init(&init_mutex, NULL);
     pthread_mutex_init(&counter_mutex, NULL);
     pthread_cond_init(&count_threshhold, NULL);
 
-    /* Count the number of tables. This number denotes the total number
-       of threads spawn.
-    */
-    save_argv= argv;
-    for (table_count= 0; *argv != NULL; argv++)
-      table_count++;
-    argv= save_argv;
-
-    if (!(worker_threads= (pthread_t*) my_malloc(table_count *
-                                                 sizeof(*worker_threads),
-                                                 MYF(0))))
-      return -2;
-
     for (counter= 0; *argv != NULL; argv++) /* Loop through tables */
     {
       pthread_mutex_lock(&counter_mutex);
@@ -680,16 +661,15 @@ int main(int argc, char **argv)
       counter++;
       pthread_mutex_unlock(&counter_mutex);
       /* now create the thread */
-      if (pthread_create(&worker_threads[worker_thread_count], &attr,
-                         worker_thread, (void *)*argv) != 0)
+      if (pthread_create(&mainthread, &attr, worker_thread, 
+                         (void *)*argv) != 0)
       {
         pthread_mutex_lock(&counter_mutex);
         counter--;
         pthread_mutex_unlock(&counter_mutex);
-        fprintf(stderr,"%s: Could not create thread\n", my_progname);
-        continue;
+        fprintf(stderr,"%s: Could not create thread\n",
+                my_progname);
       }
-      worker_thread_count++;
     }
 
     /*
@@ -704,18 +684,9 @@ int main(int argc, char **argv)
       pthread_cond_timedwait(&count_threshhold, &counter_mutex, &abstime);
     }
     pthread_mutex_unlock(&counter_mutex);
-    pthread_mutex_destroy(&init_mutex);
     pthread_mutex_destroy(&counter_mutex);
     pthread_cond_destroy(&count_threshhold);
     pthread_attr_destroy(&attr);
-
-    for(i= 0; i < worker_thread_count; i++)
-    {
-      if (pthread_join(worker_threads[i], NULL))
-        fprintf(stderr,"%s: Could not join worker thread.\n", my_progname);
-    }
-
-    my_free(worker_threads);
   }
   else
   {
