@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 /*
@@ -26,6 +26,7 @@
 #include <my_sys.h>
 #include <m_string.h>
 #include <my_getopt.h>
+#include <my_default.h>
 #include <mysql_version.h>
 
 #define load_default_groups mysqld_groups
@@ -33,6 +34,7 @@
 #undef load_default_groups
 
 my_bool opt_mysqld;
+
 const char *config_file="my";			/* Default config file */
 uint verbose= 0, opt_defaults_file_used= 0;
 const char *default_dbug_option="d:t:o,/tmp/my_print_defaults.trace";
@@ -97,16 +99,12 @@ static struct my_option my_long_options[] =
 };
 
 
-static void version()
+static void usage(my_bool version)
 {
-  printf("%s  Ver 1.6 for %s at %s\n",my_progname,SYSTEM_TYPE, MACHINE_TYPE);
-}
-
-
-static void usage() __attribute__ ((noreturn));
-static void usage()
-{
-  version();
+  printf("%s  Ver 1.6 for %s at %s\n",my_progname,SYSTEM_TYPE,
+	 MACHINE_TYPE);
+  if (version)
+    return;
   puts("This software comes with ABSOLUTELY NO WARRANTY. This is free software,\nand you are welcome to modify and redistribute it under the GPL license\n");
   puts("Prints all arguments that is give to some program using the default files");
   printf("Usage: %s [OPTIONS] [groups]\n", my_progname);
@@ -130,13 +128,12 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
       exit(0);
     case 'I':
     case '?':
-      usage();
+      usage(0);
     case 'v':
       verbose++;
       break;
     case 'V':
-      version();
-      /* fall through */
+      usage(1);
     case '#':
       DBUG_PUSH(argument ? argument : default_dbug_option);
       break;
@@ -184,7 +181,7 @@ int main(int argc, char **argv)
     nargs+= array_elements(mysqld_groups);
 
   if (nargs < 2)
-    usage();
+    usage(0);
 
   load_default_groups=(char**) my_malloc(nargs*sizeof(char*), MYF(MY_WME));
   if (!load_default_groups)
