@@ -1,5 +1,4 @@
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates.
-   Copyright (c) 2009, 2018, MariaDB Corporation
+/* Copyright (c) 2000, 2014, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /*
   More functions to be used with IO_CACHE files
@@ -45,7 +44,7 @@
 
   RETURN VALUE
     0  All OK
-    1  An error occured
+    1  An error occurred
 */
 int
 my_b_copy_to_file(IO_CACHE *cache, FILE *file)
@@ -103,14 +102,14 @@ my_off_t my_b_append_tell(IO_CACHE* info)
   */
   {
     volatile my_off_t save_pos;
-    save_pos = mysql_file_tell(info->file,MYF(0));
-    mysql_file_seek(info->file,(my_off_t)0,MY_SEEK_END,MYF(0));
+    save_pos= mysql_file_tell(info->file, MYF(0));
+    mysql_file_seek(info->file, 0, MY_SEEK_END, MYF(0));
     /*
       Save the value of my_tell in res so we can see it when studying coredump
     */
     DBUG_ASSERT(info->end_of_file - (info->append_read_pos-info->write_buffer)
-		== (res=mysql_file_tell(info->file,MYF(0))));
-    mysql_file_seek(info->file,save_pos,MY_SEEK_SET,MYF(0));
+		== (res= mysql_file_tell(info->file, MYF(0))));
+    mysql_file_seek(info->file, save_pos, MY_SEEK_SET, MYF(0));
   }
 #endif  
   res = info->end_of_file + (info->write_pos-info->append_read_pos);
@@ -204,7 +203,7 @@ size_t my_b_fill(IO_CACHE *info)
 
   if (info->seek_not_done)
   {					/* File touched, do seek */
-    if (mysql_file_seek(info->file,pos_in_file,MY_SEEK_SET,MYF(0)) ==
+    if (mysql_file_seek(info->file, pos_in_file, MY_SEEK_SET, MYF(0)) ==
 	MY_FILEPOS_ERROR)
     {
       info->error= 0;
@@ -224,7 +223,7 @@ size_t my_b_fill(IO_CACHE *info)
   }
   DBUG_EXECUTE_IF ("simulate_my_b_fill_error",
                    {DBUG_SET("+d,simulate_file_read_error");});
-  if ((length= mysql_file_read(info->file,info->buffer,max_length,
+  if ((length= mysql_file_read(info->file, info->buffer, max_length,
                        info->myflags)) == (size_t) -1)
   {
     info->error= -1;
@@ -288,7 +287,7 @@ my_off_t my_b_filelength(IO_CACHE *info)
     return my_b_tell(info);
 
   info->seek_not_done= 1;
-  return mysql_file_seek(info->file, 0L, MY_SEEK_END, MYF(0));
+  return mysql_file_seek(info->file, 0, MY_SEEK_END, MYF(0));
 }
 
 
@@ -505,7 +504,7 @@ process_flags:
       if (my_b_write(info, (uchar*) buff, length2))
 	goto err;
     }
-    else if ((*fmt == 'l' && fmt[1] == 'd') || fmt[1] == 'u')
+    else if ((*fmt == 'l' && (fmt[1] == 'd' || fmt[1] == 'u')))
       /* long parameter */
     {
       register long iarg;
