@@ -13,9 +13,10 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1335  USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
 
 #include <my_global.h>
+#include <my_default.h>
 #include <m_string.h>
 #include <stdlib.h>
 #include <my_sys.h>
@@ -1314,6 +1315,8 @@ void my_print_help(const struct my_option *options)
 
   for (optp= options; optp->name; optp++)
   {
+    if (!optp->comment)
+      continue;
     if (optp->id && optp->id < 256)
     {
       printf("  -%c%s", optp->id, strlen(optp->name) ? ", " : "  ");
