@@ -11,11 +11,11 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 
 #define PLUGIN_VERSION 0x104
-#define PLUGIN_STR_VERSION "1.4.4"
+#define PLUGIN_STR_VERSION "1.4.0"
 
 #define _my_thread_var loc_thread_var
 
@@ -97,13 +97,11 @@ static void closelog() {}
 #define FLOGGER_NO_PSI
 
 /* How to access the pthread_mutex in mysql_mutex_t */
-//#ifdef SAFE_MUTEX
-//#define mysql_mutex_real_mutex(A) &(A)->m_mutex.mutex
-//#elif defined(MY_PTHREAD_FASTMUTEX)
-//#define mysql_mutex_real_mutex(A) &(A)->m_mutex.mutex
-//#else
+#if defined(SAFE_MUTEX) || defined(MY_PTHREAD_FASTMUTEX)
+#define mysql_mutex_real_mutex(A) &(A)->m_mutex.mutex
+#else
 #define mysql_mutex_real_mutex(A) &(A)->m_mutex
-//#endif
+#endif
 
 #define flogger_mutex_init(A,B,C) do{}while(0)
 #define flogger_mutex_destroy(A) do{}while(0)
@@ -158,8 +156,10 @@ static File loc_open(const char *FileName, int Flags)
   File fd;
 #if defined(_WIN32)
   fd= my_win_open(FileName, Flags);
+#elif !defined(NO_OPEN_3)
+  fd = open(FileName, Flags, my_umask);     /* Normal unix */
 #else
-  fd = open(FileName, Flags, my_umask);
+  fd = open((char *) FileName, Flags);
 #endif
   my_errno= errno;
   return fd;
@@ -366,17 +366,16 @@ static MYSQL_SYSVAR_STR(excl_users, excl_users, PLUGIN_VAR_RQCMDARG,
 /* bits in the event filter. */
 #define EVENT_CONNECT 1
 #define EVENT_QUERY_ALL 2
-#define EVENT_QUERY 122
+#define EVENT_QUERY 58
 #define EVENT_TABLE 4
 #define EVENT_QUERY_DDL 8
 #define EVENT_QUERY_DML 16
 #define EVENT_QUERY_DCL 32
-#define EVENT_QUERY_DML_NO_SELECT 64
 
 static const char *event_names[]=
 {
   "CONNECT", "QUERY", "TABLE", "QUERY_DDL", "QUERY_DML", "QUERY_DCL",
-  "QUERY_DML_NO_SELECT", NULL
+  NULL
 };
 static TYPELIB events_typelib=
 {
@@ -384,7 +383,7 @@ static TYPELIB events_typelib=
 };
 static MYSQL_SYSVAR_SET(events, events, PLUGIN_VAR_RQCMDARG,
        "Specifies the set of events to monitor. Can be CONNECT, QUERY, TABLE,"
-           " QUERY_DDL, QUERY_DML, QUERY_DML_NO_SELECT, QUERY_DCL.",
+           " QUERY_DDL, QUERY_DML, QUERY_DCL.",
        NULL, NULL, 0, &events_typelib);
 #define OUTPUT_SYSLOG 0
 #define OUTPUT_FILE 1
@@ -428,8 +427,9 @@ static MYSQL_SYSVAR_UINT(query_log_limit, query_log_limit,
 char locinfo_ini_value[sizeof(struct connection_info)+4];
 
 static MYSQL_THDVAR_STR(loc_info,
-                        PLUGIN_VAR_NOSYSVAR | PLUGIN_VAR_NOCMDOPT | PLUGIN_VAR_MEMALLOC,
-                        "Internal info", NULL, NULL, locinfo_ini_value);
+                        PLUGIN_VAR_READONLY | PLUGIN_VAR_MEMALLOC,
+                        "Auxiliary info.", NULL, NULL,
+                        locinfo_ini_value);
 
 static const char *syslog_facility_names[]=
 {
@@ -858,21 +858,6 @@ struct sa_keyword dml_keywords[]=
 };
 
 
-struct sa_keyword dml_no_select_keywords[]=
-{
-  {2, "DO", 0, SQLCOM_DML},
-  {4, "CALL", 0, SQLCOM_DML},
-  {4, "LOAD", &data_word, SQLCOM_DML},
-  {4, "LOAD", &xml_word, SQLCOM_DML},
-  {6, "DELETE", 0, SQLCOM_DML},
-  {6, "INSERT", 0, SQLCOM_DML},
-  {6, "UPDATE", 0, SQLCOM_DML},
-  {7, "HANDLER", 0, SQLCOM_DML},
-  {7, "REPLACE", 0, SQLCOM_DML},
-  {0, NULL, 0, SQLCOM_DML}
-};
-
-
 struct sa_keyword dcl_keywords[]=
 {
   {6, "CREATE", &user_word, SQLCOM_DCL},
@@ -1106,7 +1091,6 @@ static void setup_connection_connect(struct connection_info *cn,
     const struct mysql_event_connection *event)
 {
   cn->query_id= 0;
-  cn->query_length= 0;
   cn->log_always= 0;
   cn->thread_id= event->thread_id;
   get_str_n(cn->db, &cn->db_length, sizeof(cn->db),
@@ -1138,21 +1122,6 @@ do { \
 } while(0)
 
 
-#define ESC_MAP_SIZE 0x60
-static const char esc_map[ESC_MAP_SIZE]=
-{
-  0, 0, 0, 0, 0, 0, 0, 0, 'b', 't', 'n', 0, 'f', 'r', 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, '\'', 0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, '\\', 0, 0, 0
-};
-
-static char escaped_char(char c)
-{
-  return ((unsigned char ) c) >= ESC_MAP_SIZE ? 0 : esc_map[(unsigned char) c];
-}
 
 
 static void setup_connection_initdb(struct connection_info *cn,
@@ -1163,7 +1132,6 @@ static void setup_connection_initdb(struct connection_info *cn,
 
   cn->thread_id= event->general_thread_id;
   cn->query_id= 0;
-  cn->query_length= 0;
   cn->log_always= 0;
   get_str_n(cn->db, &cn->db_length, sizeof(cn->db),
             event->general_query, event->general_query_length);
@@ -1196,7 +1164,6 @@ static void setup_connection_table(struct connection_info *cn,
   cn->thread_id= event->thread_id;
   cn->query_id= query_counter++;
   cn->log_always= 0;
-  cn->query_length= 0;
   get_str_n(cn->db, &cn->db_length, sizeof(cn->db),
             event->database, event->database_length);
   get_str_n(cn->user, &cn->user_length, sizeof(cn->db),
@@ -1218,7 +1185,6 @@ static void setup_connection_query(struct connection_info *cn,
   cn->thread_id= event->general_thread_id;
   cn->query_id= query_counter++;
   cn->log_always= 0;
-  cn->query_length= 0;
   get_str_n(cn->db, &cn->db_length, sizeof(cn->db), "", 0);
 
   if (get_user_host(event->general_user, event->general_user_length,
@@ -1359,16 +1325,21 @@ static size_t escape_string(const char *str, unsigned int len,
   const char *res_end= result + result_len - 2;
   while (len)
   {
-    char esc_c;
-
     if (result >= res_end)
       break;
-    if ((esc_c= escaped_char(*str)))
+    if (*str == '\'')
     {
       if (result+1 >= res_end)
         break;
       *(result++)= '\\';
-      *(result++)= esc_c;
+      *(result++)= '\'';
+    }
+    else if (*str == '\\')
+    {
+      if (result+1 >= res_end)
+        break;
+      *(result++)= '\\';
+      *(result++)= '\\';
     }
     else if (is_space(*str))
       *(result++)= ' ';
@@ -1457,12 +1428,19 @@ static size_t escape_string_hide_passwords(const char *str, unsigned int len,
 no_password:
     if (result >= res_end)
       break;
-    if ((b_char= escaped_char(*str)))
+    if (*str == '\'')
     {
       if (result+1 >= res_end)
         break;
       *(result++)= '\\';
-      *(result++)= b_char;
+      *(result++)= '\'';
+    }
+    else if (*str == '\\')
+    {
+      if (result+1 >= res_end)
+        break;
+      *(result++)= '\\';
+      *(result++)= '\\';
     }
     else if (is_space(*str))
       *(result++)= ' ';
@@ -1653,11 +1631,6 @@ static int log_statement_ex(const struct connection_info *cn,
       if (filter_query_type(query, dml_keywords))
         goto do_log_query;
     }
-    if (events & EVENT_QUERY_DML_NO_SELECT)
-    {
-      if (filter_query_type(query, dml_no_select_keywords))
-        goto do_log_query;
-    }
     if (events & EVENT_QUERY_DCL)
     {
       if (filter_query_type(query, dcl_keywords))
@@ -2036,7 +2009,6 @@ void auditing(MYSQL_THD thd, unsigned int event_class, const void *ev)
         event_query_command(event))
     {
       log_statement(cn, event, "QUERY");
-      cn->query_length= 0; /* So the log_current_query() won't log this again. */
     }
   }
   else if (event_class == MYSQL_AUDIT_TABLE_CLASS && FILTER(EVENT_TABLE) && cn)
@@ -2324,10 +2296,10 @@ typedef struct loc_system_variables
 } LOC_SV;
 
 
-static int init_done= 0;
-
 static int server_audit_init(void *p __attribute__((unused)))
 {
+  const void *my_hash_init_ptr;
+
   if (!serv_ver)
   {
 #ifdef _WIN32
@@ -2336,16 +2308,11 @@ static int server_audit_init(void *p __attribute__((unused)))
     serv_ver= server_version;
 #endif /*_WIN32*/
   }
-  if (!mysql_57_started)
+  my_hash_init_ptr= dlsym(RTLD_DEFAULT, "_my_hash_init");
+  if (!my_hash_init_ptr)
   {
-    const void *my_hash_init_ptr= dlsym(RTLD_DEFAULT, "_my_hash_init");
-    if (!my_hash_init_ptr)
-    {
-      maria_above_5= 1;
-      my_hash_init_ptr= dlsym(RTLD_DEFAULT, "my_hash_init2");
-    }
-    if (!my_hash_init_ptr)
-      return 1;
+    maria_above_5= 1;
+    my_hash_init_ptr= dlsym(RTLD_DEFAULT, "my_hash_init2");
   }
 
   if(!(int_mysql_data_home= dlsym(RTLD_DEFAULT, "mysql_data_home")))
@@ -2354,7 +2321,7 @@ static int server_audit_init(void *p __attribute__((unused)))
       int_mysql_data_home= &default_home;
   }
 
-  if (!serv_ver)
+  if (!serv_ver || !my_hash_init_ptr)
     return 1;
 
   if (!started_mysql)
@@ -2434,7 +2401,6 @@ static int server_audit_init(void *p __attribute__((unused)))
   if (logging)
     start_logging();
 
-  init_done= 1;
   return 0;
 }
 
@@ -2450,10 +2416,6 @@ static int server_audit_init_mysql(void *p)
 
 static int server_audit_deinit(void *p __attribute__((unused)))
 {
-  if (!init_done)
-    return 0;
-
-  init_done= 0;
   coll_free(&incl_user_coll);
   coll_free(&excl_user_coll);
 
@@ -2552,8 +2514,7 @@ static void log_current_query(MYSQL_THD thd)
   if (!thd)
     return;
   cn= get_loc_info(thd);
-  if (!ci_needs_setup(cn) && cn->query_length &&
-      FILTER(EVENT_QUERY) && do_log_user(cn->user))
+  if (!ci_needs_setup(cn) && FILTER(EVENT_QUERY) && do_log_user(cn->user))
   {
     log_statement_ex(cn, cn->query_time, thd_get_thread_id(thd),
         cn->query, cn->query_length, 0, "QUERY");
@@ -2877,15 +2838,12 @@ void __attribute__ ((constructor)) audit_plugin_so_init(void)
       if (sc >= 24)
         use_event_data_for_disconnect= 1;
     }
-    else if ((serv_ver[0] == '5' && serv_ver[2] == '7') ||
-             (serv_ver[0] == '8' && serv_ver[2] == '0'))
+    else if (serv_ver[0] == '5' && serv_ver[2] == '7')
     {
       mysql_57_started= 1;
       _mysql_plugin_declarations_[0].info= mysql_v4_descriptor;
       use_event_data_for_disconnect= 1;
     }
-    MYSQL_SYSVAR_NAME(loc_info).flags= PLUGIN_VAR_STR | PLUGIN_VAR_THDLOCAL |
-      PLUGIN_VAR_READONLY | PLUGIN_VAR_MEMALLOC;
   }
 
   memset(locinfo_ini_value, 'O', sizeof(locinfo_ini_value)-1);
