@@ -12,8 +12,9 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
+#include <my_global.h>
 #include "sql_priv.h"
 /*
   It is necessary to include set_var.h instead of item.h because there
@@ -44,17 +45,10 @@ Item_row::Item_row(List<Item> &arg):
 
   //TODO: think placing 2-3 component items in item (as it done for function)
   if ((arg_count= arg.elements))
-  {
     items= (Item**) sql_alloc(sizeof(Item*)*arg_count);
-    if (!items)
-    {
-      arg_count= 0;
-      return;
-    }
-  }
   else
     items= 0;
-  List_iterator<Item> li(arg);
+  List_iterator_fast<Item> li(arg);
   uint i= 0;
   Item *item;
   while ((item= li++))
@@ -64,27 +58,6 @@ Item_row::Item_row(List<Item> &arg):
   }
 }
 
-
-Item_row::Item_row(Item *item):
-  Item(),
-  used_tables_cache(0),
-  not_null_tables_cache(0),
-  arg_count(item->cols()),
-  const_item_cache(1),
-  with_null(0)
-{
-  items= (Item**) sql_alloc(sizeof(Item*) * arg_count);
-  if (!items)
-  {
-    arg_count= 0;
-    return;
-  }
-  for (uint i= 0; i < arg_count; i++)
-  {
-    items[i]= item->element_index(i);
-  }
-}
-
 void Item_row::illegal_method_call(const char *method)
 {
   DBUG_ENTER("Item_row::illegal_method_call");
@@ -125,7 +98,6 @@ bool Item_row::fix_fields(THD *thd, Item **ref)
     with_sum_func= with_sum_func || item->with_sum_func;
     with_field= with_field || item->with_field;
     with_subselect|= item->with_subselect;
-    with_param|= item->with_param;
   }
   fixed= 1;
   return FALSE;
