@@ -24,7 +24,6 @@
 #include <m_string.h>
 
 #include <windows.h>
-#include <assert.h>
 #include <shellapi.h>
 #include <accctrl.h>
 #include <aclapi.h>
@@ -37,7 +36,7 @@
 "Usage: mysql_install_db.exe [OPTIONS]\n" \
 "OPTIONS:"
 
-extern "C" const char mysql_bootstrap_sql[];
+extern "C" const char* mysql_bootstrap_sql[];
 
 char default_os_user[]= "NT AUTHORITY\\NetworkService";
 static int create_db_instance();
@@ -120,10 +119,10 @@ static void die(const char *fmt, ...)
   if (verbose_errors)
   {
    fprintf(stderr,
-   "http://kb.askmonty.org/v/installation-issues-on-windows contains some help\n"
+   "https://mariadb.com/kb/en/installation-issues-on-windows contains some help\n"
    "for solving the most common problems.  If this doesn't help you, please\n"
-   "leave a comment in the Knowledgebase or file a bug report at\n"
-   "http://mariadb.org/jira");
+   "leave a comment in the Knowledge Base or file a bug report at\n"
+   "https://jira.mariadb.org");
   }
   fflush(stderr);
   va_end(args);
@@ -250,7 +249,7 @@ static char *init_bootstrap_command_line(char *cmdline, size_t size)
     "\"\"%s\" --no-defaults %s --bootstrap"
     " \"--lc-messages-dir=%s/share\""
     " --basedir=. --datadir=. --default-storage-engine=myisam"
-    " --max_allowed_packet=9M --loose-skip-innodb"
+    " --max_allowed_packet=9M "
     " --net-buffer-length=16k\"", mysqld_path,
     opt_verbose_bootstrap?"--console":"", basedir );
   return cmdline;
@@ -564,6 +563,10 @@ static int create_db_instance()
   if (!in)
     goto end;
 
+  if (setvbuf(in, NULL, _IONBF, 0))
+  {
+    verbose("WARNING: Cannot disable buffering on mysqld's stdin");
+  }
   if (fwrite("use mysql;\n",11,1, in) != 1)
   {
     verbose("ERROR: Cannot write to mysqld's stdin");
@@ -571,12 +574,16 @@ static int create_db_instance()
     goto end;
   }
 
-  /* Write the bootstrap script to stdin. */
-  if (fwrite(mysql_bootstrap_sql, strlen(mysql_bootstrap_sql), 1, in) != 1)
+  int i;
+  for (i=0; mysql_bootstrap_sql[i]; i++)
   {
-    verbose("ERROR: Cannot write to mysqld's stdin");
-    ret= 1;
-    goto end;
+    /* Write the bootstrap script to stdin. */
+    if (fwrite(mysql_bootstrap_sql[i], strlen(mysql_bootstrap_sql[i]), 1, in) != 1)
+    {
+      verbose("ERROR: Cannot write to mysqld's stdin");
+      ret= 1;
+      goto end;
+    }
   }
 
   /* Remove default user, if requested. */
@@ -627,6 +634,14 @@ static int create_db_instance()
     goto end;
   }
 
+  /* 
+    Remove innodb log files if they exist (this works around "different size logs" 
+    error in MSI installation). TODO : remove this with the next Innodb, where
+    different size is handled gracefully.
+  */
+  DeleteFile("ib_logfile0");
+  DeleteFile("ib_logfile1");
+
   /* Create my.ini file in data directory.*/
   ret= create_myini();
   if (ret)
