@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /* 
  * sql_manager.cc
@@ -21,9 +21,9 @@
  *   o Berkeley DB: removing unneeded log files.
  */
 
+#include <my_global.h>
 #include "sql_priv.h"
 #include "sql_manager.h"
-#include "unireg.h"                    // REQUIRED: for other includes
 #include "sql_base.h"                           // flush_tables
 
 static bool volatile manager_thread_in_use;
@@ -110,7 +110,7 @@ pthread_handler_t handle_manager(void *arg __attribute__((unused)))
 
     if (error == ETIMEDOUT || error == ETIME)
     {
-      tdc_flush_unused_tables();
+      tc_purge();
       error = 0;
       reset_flush_time = TRUE;
     }
