@@ -30,11 +30,11 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #define MYSQL_SERVER 1
 #include <my_global.h>
 #include "sql_priv.h"
-#include <mysql/plugin.h>
 
 #include "ha_federatedx.h"
 
 #include "m_string.h"
+#include "mysqld_error.h"
 #include "sql_servers.h"
 
 #ifdef USE_PRAGMA_IMPLEMENTATION
@@ -138,7 +138,7 @@ federatedx_io_mysql::federatedx_io_mysql(FEDERATEDX_SERVER *aserver)
   bzero(&mysql, sizeof(MYSQL));
   bzero(&savepoints, sizeof(DYNAMIC_ARRAY));
 
-  my_init_dynamic_array(&savepoints, sizeof(SAVEPT), 16, 16);  
+  my_init_dynamic_array(&savepoints, sizeof(SAVEPT), 16, 16, MYF(0));
   
   DBUG_VOID_RETURN;
 }
@@ -263,8 +263,9 @@ ulong federatedx_io_mysql::savepoint_release(ulong sp)
     savept= dynamic_element(&savepoints, savepoints.elements - 1, SAVEPT *);
     if (savept->level < sp)
       break;
-    if ((savept->flags & (SAVEPOINT_REALIZED | SAVEPOINT_RESTRICT)) == SAVEPOINT_REALIZED)
-      last= savept;
+  if ((savept->flags & (SAVEPOINT_REALIZED | 
+                        SAVEPOINT_RESTRICT)) == SAVEPOINT_REALIZED)
+    last= savept;
     savepoints.elements--;
   }
 
@@ -290,8 +291,8 @@ ulong federatedx_io_mysql::savepoint_rollback(ulong sp)
   while (savepoints.elements)
   {
     savept= dynamic_element(&savepoints, savepoints.elements - 1, SAVEPT *);
-    if (savept->level <= sp)
-      break;
+  if (savept->level <= sp)
+    break;
     savepoints.elements--;
   }
 
@@ -424,8 +425,10 @@ int federatedx_io_mysql::actual_query(const char *buffer, uint length)
 
   if (!mysql.net.vio)
   {
+    my_bool my_true= 1;
+
     if (!(mysql_init(&mysql)))
-    DBUG_RETURN(-1);
+      DBUG_RETURN(-1);
   
     /*
 	BUG# 17044 Federated Storage Engine is not UTF8 clean
@@ -434,6 +437,8 @@ int federatedx_io_mysql::actual_query(const char *buffer, uint length)
     */
     /* this sets the csname like 'set names utf8' */
     mysql_options(&mysql, MYSQL_SET_CHARSET_NAME, get_charsetname());
+    mysql_options(&mysql, MYSQL_OPT_USE_THREAD_SPECIFIC_MEMORY,
+                  (char*) &my_true);
 
     if (!mysql_real_connect(&mysql,
                             get_hostname(),
