@@ -1,6 +1,6 @@
 /*
-   Copyright (c) 2000, 2018, Oracle and/or its affiliates.
-   Copyright (c) 2009, 2017, MariaDB Corporation.
+   Copyright (c) 2000, 2012, Oracle and/or its affiliates.
+   Copyright (c) 2009, 2014, SkySQL Ab.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 
 #ifdef USE_PRAGMA_IMPLEMENTATION
@@ -21,9 +21,9 @@
 #endif
 
 #define MYSQL_SERVER 1
+#include "sql_plugin.h"
 #include "sql_priv.h"
 #include "key.h"                                // key_copy
-#include "sql_plugin.h"
 #include <m_ctype.h>
 #include <my_bit.h>
 #include "ha_myisam.h"
@@ -79,7 +79,7 @@ static MYSQL_THDVAR_ULONG(repair_threads, PLUGIN_VAR_RQCMDARG,
 static MYSQL_THDVAR_ULONGLONG(sort_buffer_size, PLUGIN_VAR_RQCMDARG,
   "The buffer that is allocated when sorting the index when doing "
   "a REPAIR or when creating indexes with CREATE INDEX or ALTER TABLE", NULL, NULL,
-  8192 * 1024, MIN_SORT_BUFFER + MALLOC_OVERHEAD, SIZE_T_MAX, 1);
+  SORT_BUFFER_INIT, MIN_SORT_BUFFER, SIZE_T_MAX, 1);
 
 static MYSQL_SYSVAR_BOOL(use_mmap, opt_myisam_use_mmap, PLUGIN_VAR_NOCMDARG,
   "Use memory mapping for reading and writing MyISAM tables", NULL, NULL, FALSE);
@@ -141,6 +141,9 @@ static void mi_check_print_msg(HA_CHECK *param,	const char* msg_type,
   char msgbuf[MYSQL_ERRMSG_SIZE];
   char name[NAME_LEN*2+2];
 
+  if (param->testflag & T_SUPPRESS_ERR_HANDLING)
+    return;
+
   msg_length= my_vsnprintf(msgbuf, sizeof(msgbuf), fmt, args);
   msgbuf[sizeof(msgbuf) - 1] = 0; // healthy paranoia
 
@@ -246,8 +249,8 @@ int table2myisam(TABLE *table_arg, MI_KEYDEF **keydef_out,
       pos->algorithm;
     keydef[i].block_length= pos->block_size;
     keydef[i].seg= keyseg;
-    keydef[i].keysegs= pos->key_parts;
-    for (j= 0; j < pos->key_parts; j++)
+    keydef[i].keysegs= pos->user_defined_key_parts;
+    for (j= 0; j < pos->user_defined_key_parts; j++)
     {
       Field *field= pos->key_part[j].field;
       type= field->key_type();
@@ -299,7 +302,7 @@ int table2myisam(TABLE *table_arg, MI_KEYDEF **keydef_out,
         keydef[i].seg[j].flag|= HA_BLOB_PART;
         /* save number of bytes used to pack length */
         keydef[i].seg[j].bit_start= (uint) (field->pack_length() -
-                                            share->blob_ptr_size);
+                                            portable_sizeof_char_ptr);
       }
       else if (field->type() == MYSQL_TYPE_BIT)
       {
@@ -309,7 +312,7 @@ int table2myisam(TABLE *table_arg, MI_KEYDEF **keydef_out,
                                           (uchar*) table_arg->record[0]);
       }
     }
-    keyseg+= pos->key_parts;
+    keyseg+= pos->user_defined_key_parts;
   }
   if (table_arg->found_next_number_field)
     keydef[share->next_number_index].flag|= HA_AUTO_KEY;
@@ -353,12 +356,23 @@ int table2myisam(TABLE *table_arg, MI_KEYDEF **keydef_out,
 
     if (found->flags & BLOB_FLAG)
       recinfo_pos->type= FIELD_BLOB;
-    else if (found->type() == MYSQL_TYPE_TIMESTAMP)
+    else if (found->real_type() == MYSQL_TYPE_TIMESTAMP)
+    {
+      /* pre-MySQL-5.6.4 TIMESTAMP, or MariaDB-5.3+ TIMESTAMP */
       recinfo_pos->type= FIELD_NORMAL;
+    }
     else if (found->type() == MYSQL_TYPE_VARCHAR)
       recinfo_pos->type= FIELD_VARCHAR;
     else if (!(options & HA_OPTION_PACK_RECORD))
       recinfo_pos->type= FIELD_NORMAL;
+    else if (found->real_type() == MYSQL_TYPE_TIMESTAMP2)
+    {
+      /*
+        MySQL-5.6.4+ erroneously marks Field_timestampf as FIELD_SKIP_PRESPACE,
+        but only if HA_OPTION_PACK_RECORD is set.
+      */
+      recinfo_pos->type= FIELD_SKIP_PRESPACE;
+    }
     else if (found->zero_pack())
       recinfo_pos->type= FIELD_SKIP_ZERO;
     else
@@ -466,8 +480,8 @@ int check_definition(MI_KEYDEF *t1_keyinfo, MI_COLUMNDEF *t1_recinfo,
     {
        DBUG_PRINT("error", ("Key %d has different definition", i));
        DBUG_PRINT("error", ("t1_fulltext= %d, t2_fulltext=%d",
-                            test(t1_keyinfo[i].flag & HA_FULLTEXT),
-                            test(t2_keyinfo[i].flag & HA_FULLTEXT)));
+                            MY_TEST(t1_keyinfo[i].flag & HA_FULLTEXT),
+                            MY_TEST(t2_keyinfo[i].flag & HA_FULLTEXT)));
        DBUG_RETURN(1);
     }
     if (t1_keyinfo[i].flag & HA_SPATIAL && t2_keyinfo[i].flag & HA_SPATIAL)
@@ -477,8 +491,8 @@ int check_definition(MI_KEYDEF *t1_keyinfo, MI_COLUMNDEF *t1_recinfo,
     {
        DBUG_PRINT("error", ("Key %d has different definition", i));
        DBUG_PRINT("error", ("t1_spatial= %d, t2_spatial=%d",
-                            test(t1_keyinfo[i].flag & HA_SPATIAL),
-                            test(t2_keyinfo[i].flag & HA_SPATIAL)));
+                            MY_TEST(t1_keyinfo[i].flag & HA_SPATIAL),
+                            MY_TEST(t2_keyinfo[i].flag & HA_SPATIAL)));
        DBUG_RETURN(1);
     }
     if ((!mysql_40_compat &&
@@ -573,6 +587,8 @@ void mi_check_print_error(HA_CHECK *param, const char *fmt,...)
 {
   param->error_printed|=1;
   param->out_flag|= O_DATA_LOST;
+  if (param->testflag & T_SUPPRESS_ERR_HANDLING)
+    return;
   va_list args;
   va_start(args, fmt);
   mi_check_print_msg(param, "error", fmt, args);
@@ -631,8 +647,10 @@ void _mi_report_crashed(MI_INFO *file, const char *message,
   for (element= file->s->in_use; element; element= list_rest(element))
   {
     THD *thd= (THD*) element->data;
-    sql_print_error("%s", thd ? thd_security_context(thd, buf, sizeof(buf), 0)
-                              : "Unknown thread accessing table");
+    sql_print_error("%s",
+                    thd ?
+                    thd_get_error_context_description(thd, buf, sizeof(buf), 0)
+                    : "Unknown thread accessing table");
   }
   mysql_mutex_unlock(&file->s->intern_lock);
 }
@@ -650,7 +668,7 @@ ha_myisam::ha_myisam(handlerton *hton, TABLE_SHARE *table_arg)
   :handler(hton, table_arg), file(0),
   int_table_flags(HA_NULL_IN_KEY | HA_CAN_FULLTEXT | HA_CAN_SQL_HANDLER |
                   HA_BINLOG_ROW_CAPABLE | HA_BINLOG_STMT_CAPABLE |
-                  HA_CAN_VIRTUAL_COLUMNS |
+                  HA_CAN_VIRTUAL_COLUMNS | HA_CAN_EXPORT |
                   HA_DUPLICATE_POS | HA_CAN_INDEX_BLOBS | HA_AUTO_PART_KEY |
                   HA_FILE_BASED | HA_CAN_GEOMETRY | HA_NO_TRANSACTIONS |
                   HA_CAN_INSERT_DELAYED | HA_CAN_BIT_FIELD | HA_CAN_RTREEKEYS |
@@ -674,12 +692,6 @@ static const char *ha_myisam_exts[] = {
   NullS
 };
 
-const char **ha_myisam::bas_ext() const
-{
-  return ha_myisam_exts;
-}
-
-
 const char *ha_myisam::index_type(uint key_number)
 {
   return ((table->key_info[key_number].flags & HA_FULLTEXT) ? 
@@ -730,8 +742,8 @@ int ha_myisam::open(const char *name, int mode, uint test_if_locked)
     growing files. Using an open_flag instead of calling mi_extra(...
     HA_EXTRA_MMAP ...) after mi_open() has the advantage that the
     mapping is not repeated for every open, but just done on the initial
-    open, when the MyISAM share is created. Every time the server
-    requires opening a new instance of a table it calls this method. We
+    open, when the MyISAM share is created. Everytime the server
+    requires to open a new instance of a table it calls this method. We
     will always supply HA_OPEN_MMAP for a permanent table. However, the
     MyISAM storage engine will ignore this flag if this is a secondary
     open of a table that is in use by other threads already (if the
@@ -811,7 +823,15 @@ int ha_myisam::open(const char *name, int mode, uint test_if_locked)
     table->key_info[i].block_size= file->s->keyinfo[i].block_length;
   }
   my_errno= 0;
+
+  /* Count statistics of usage for newly open normal files */
+  if (file->s->reopen == 1 && ! (test_if_locked & HA_OPEN_TMP_TABLE))
+  {
+    if (file->s->delay_key_write)
+      feature_files_opened_with_delayed_keys++;
+  }
   goto end;
+
  err:
   this->close();
  end:
@@ -835,10 +855,6 @@ int ha_myisam::close(void)
 
 int ha_myisam::write_row(uchar *buf)
 {
-  /* If we have a timestamp column, update it to the current time */
-  if (table->timestamp_field_type & TIMESTAMP_AUTO_SET_ON_INSERT)
-    table->timestamp_field->set_time();
-
   /*
     If we have an auto_increment column and we are writing a changed row
     or a new row, then update the auto_increment value in the record.
@@ -856,59 +872,59 @@ int ha_myisam::check(THD* thd, HA_CHECK_OPT* check_opt)
 {
   if (!file) return HA_ADMIN_INTERNAL_ERROR;
   int error;
-  HA_CHECK *param= (HA_CHECK*) thd->alloc(sizeof *param);
+  HA_CHECK &param= *(HA_CHECK*) thd->alloc(sizeof(param));
   MYISAM_SHARE* share = file->s;
   const char *old_proc_info=thd->proc_info;
 
-  if (!param)
+  if (!&param)
     return HA_ADMIN_INTERNAL_ERROR;
 
   thd_proc_info(thd, "Checking table");
-  myisamchk_init(param);
-  param->thd = thd;
-  param->op_name =   "check";
-  param->db_name=    table->s->db.str;
-  param->table_name= table->alias.c_ptr();
-  param->testflag = check_opt->flags | T_CHECK | T_SILENT;
-  param->stats_method= (enum_handler_stats_method)THDVAR(thd, stats_method);
+  myisamchk_init(&param);
+  param.thd = thd;
+  param.op_name =   "check";
+  param.db_name=    table->s->db.str;
+  param.table_name= table->alias.c_ptr();
+  param.testflag = check_opt->flags | T_CHECK | T_SILENT;
+  param.stats_method= (enum_handler_stats_method)THDVAR(thd, stats_method);
 
   if (!(table->db_stat & HA_READ_ONLY))
-    param->testflag|= T_STATISTICS;
-  param->using_global_keycache = 1;
+    param.testflag|= T_STATISTICS;
+  param.using_global_keycache = 1;
 
   if (!mi_is_crashed(file) &&
-      (((param->testflag & T_CHECK_ONLY_CHANGED) &&
+      (((param.testflag & T_CHECK_ONLY_CHANGED) &&
 	!(share->state.changed & (STATE_CHANGED | STATE_CRASHED |
 				  STATE_CRASHED_ON_REPAIR)) &&
 	share->state.open_count == 0) ||
-       ((param->testflag & T_FAST) && (share->state.open_count ==
+       ((param.testflag & T_FAST) && (share->state.open_count ==
 				      (uint) (share->global_changed ? 1 : 0)))))
     return HA_ADMIN_ALREADY_DONE;
 
-  error = chk_status(param, file);		// Not fatal
-  error = chk_size(param, file);
+  error = chk_status(&param, file);		// Not fatal
+  error = chk_size(&param, file);
   if (!error)
-    error |= chk_del(param, file, param->testflag);
+    error |= chk_del(&param, file, param.testflag);
   if (!error)
-    error = chk_key(param, file);
+    error = chk_key(&param, file);
   if (!error)
   {
-    if ((!(param->testflag & T_QUICK) &&
+    if ((!(param.testflag & T_QUICK) &&
 	 ((share->options &
 	   (HA_OPTION_PACK_RECORD | HA_OPTION_COMPRESS_RECORD)) ||
-	  (param->testflag & (T_EXTEND | T_MEDIUM)))) ||
+	  (param.testflag & (T_EXTEND | T_MEDIUM)))) ||
 	mi_is_crashed(file))
     {
-      ulonglong old_testflag= param->testflag;
-      param->testflag|=T_MEDIUM;
-      if (!(error= init_io_cache(&param->read_cache, file->dfile,
+      ulonglong old_testflag= param.testflag;
+      param.testflag|=T_MEDIUM;
+      if (!(error= init_io_cache(&param.read_cache, file->dfile,
                                  my_default_record_cache_size, READ_CACHE,
                                  share->pack.header_length, 1, MYF(MY_WME))))
       {
-        error= chk_data_link(param, file, test(param->testflag & T_EXTEND));
-        end_io_cache(&(param->read_cache));
+        error= chk_data_link(&param, file, MY_TEST(param.testflag & T_EXTEND));
+        end_io_cache(&(param.read_cache));
       }
-      param->testflag= old_testflag;
+      param.testflag= old_testflag;
     }
   }
   if (!error)
@@ -916,7 +932,7 @@ int ha_myisam::check(THD* thd, HA_CHECK_OPT* check_opt)
     if ((share->state.changed & (STATE_CHANGED |
 				 STATE_CRASHED_ON_REPAIR |
 				 STATE_CRASHED | STATE_NOT_ANALYZED)) ||
-	(param->testflag & T_STATISTICS) ||
+	(param.testflag & T_STATISTICS) ||
 	mi_is_crashed(file))
     {
       file->update|=HA_STATE_CHANGED | HA_STATE_ROW_CHANGED;
@@ -924,7 +940,7 @@ int ha_myisam::check(THD* thd, HA_CHECK_OPT* check_opt)
       share->state.changed&= ~(STATE_CHANGED | STATE_CRASHED |
 			       STATE_CRASHED_ON_REPAIR);
       if (!(table->db_stat & HA_READ_ONLY))
-	error=update_state_info(param,file,UPDATE_TIME | UPDATE_OPEN_COUNT |
+	error=update_state_info(&param,file,UPDATE_TIME | UPDATE_OPEN_COUNT |
 				UPDATE_STAT);
       mysql_mutex_unlock(&share->intern_lock);
       info(HA_STATUS_NO_LOCK | HA_STATUS_TIME | HA_STATUS_VARIABLE |
@@ -951,30 +967,30 @@ int ha_myisam::check(THD* thd, HA_CHECK_OPT* check_opt)
 int ha_myisam::analyze(THD *thd, HA_CHECK_OPT* check_opt)
 {
   int error=0;
-  HA_CHECK *param= (HA_CHECK*) thd->alloc(sizeof *param);
+  HA_CHECK &param= *(HA_CHECK*) thd->alloc(sizeof(param));
   MYISAM_SHARE* share = file->s;
 
-  if (!param)
+  if (!&param)
     return HA_ADMIN_INTERNAL_ERROR;
 
-  myisamchk_init(param);
-  param->thd = thd;
-  param->op_name=    "analyze";
-  param->db_name=    table->s->db.str;
-  param->table_name= table->alias.c_ptr();
-  param->testflag= (T_FAST | T_CHECK | T_SILENT | T_STATISTICS |
+  myisamchk_init(&param);
+  param.thd = thd;
+  param.op_name=    "analyze";
+  param.db_name=    table->s->db.str;
+  param.table_name= table->alias.c_ptr();
+  param.testflag= (T_FAST | T_CHECK | T_SILENT | T_STATISTICS |
                    T_DONT_CHECK_CHECKSUM);
-  param->using_global_keycache = 1;
-  param->stats_method= (enum_handler_stats_method)THDVAR(thd, stats_method);
+  param.using_global_keycache = 1;
+  param.stats_method= (enum_handler_stats_method)THDVAR(thd, stats_method);
 
   if (!(share->state.changed & STATE_NOT_ANALYZED))
     return HA_ADMIN_ALREADY_DONE;
 
-  error = chk_key(param, file);
+  error = chk_key(&param, file);
   if (!error)
   {
     mysql_mutex_lock(&share->intern_lock);
-    error=update_state_info(param,file,UPDATE_STAT);
+    error=update_state_info(&param,file,UPDATE_STAT);
     mysql_mutex_unlock(&share->intern_lock);
   }
   else if (!mi_is_crashed(file) && !thd->killed)
@@ -986,37 +1002,37 @@ int ha_myisam::analyze(THD *thd, HA_CHECK_OPT* check_opt)
 int ha_myisam::repair(THD* thd, HA_CHECK_OPT *check_opt)
 {
   int error;
-  HA_CHECK *param= (HA_CHECK*) thd->alloc(sizeof *param);
+  HA_CHECK &param= *(HA_CHECK*) thd->alloc(sizeof(param));
   ha_rows start_records;
 
-  if (!file || !param) return HA_ADMIN_INTERNAL_ERROR;
+  if (!file || !&param) return HA_ADMIN_INTERNAL_ERROR;
 
-  myisamchk_init(param);
-  param->thd = thd;
-  param->op_name=  "repair";
-  param->testflag= ((check_opt->flags & ~(T_EXTEND)) |
+  myisamchk_init(&param);
+  param.thd = thd;
+  param.op_name=  "repair";
+  param.testflag= ((check_opt->flags & ~(T_EXTEND)) |
                    T_SILENT | T_FORCE_CREATE | T_CALC_CHECKSUM |
                    (check_opt->flags & T_EXTEND ? T_REP : T_REP_BY_SORT));
-  param->sort_buffer_length=  THDVAR(thd, sort_buffer_size);
-  param->backup_time= check_opt->start_time;
+  param.sort_buffer_length=  THDVAR(thd, sort_buffer_size);
+  param.backup_time= check_opt->start_time;
   start_records=file->state->records;
-  while ((error=repair(thd,*param,0)) && param->retry_repair)
+  while ((error=repair(thd,param,0)) && param.retry_repair)
   {
-    param->retry_repair=0;
-    if (test_all_bits(param->testflag,
+    param.retry_repair=0;
+    if (test_all_bits(param.testflag,
 		      (uint) (T_RETRY_WITHOUT_QUICK | T_QUICK)))
     {
-      param->testflag&= ~(T_RETRY_WITHOUT_QUICK | T_QUICK);
+      param.testflag&= ~(T_RETRY_WITHOUT_QUICK | T_QUICK);
       /* Ensure we don't loose any rows when retrying without quick */
-      param->testflag|= T_SAFE_REPAIR;
+      param.testflag|= T_SAFE_REPAIR;
       sql_print_information("Retrying repair of: '%s' including modifying data file",
                             table->s->path.str);
       continue;
     }
-    param->testflag&= ~T_QUICK;
-    if ((param->testflag & T_REP_BY_SORT))
+    param.testflag&= ~T_QUICK;
+    if ((param.testflag & T_REP_BY_SORT))
     {
-      param->testflag= (param->testflag & ~T_REP_BY_SORT) | T_REP;
+      param.testflag= (param.testflag & ~T_REP_BY_SORT) | T_REP;
       sql_print_information("Retrying repair of: '%s' with keycache",
                             table->s->path.str);
       continue;
@@ -1038,22 +1054,22 @@ int ha_myisam::repair(THD* thd, HA_CHECK_OPT *check_opt)
 int ha_myisam::optimize(THD* thd, HA_CHECK_OPT *check_opt)
 {
   int error;
-  HA_CHECK *param= (HA_CHECK*) thd->alloc(sizeof *param);
+  HA_CHECK &param= *(HA_CHECK*) thd->alloc(sizeof(param));
 
-  if (!file || !param) return HA_ADMIN_INTERNAL_ERROR;
+  if (!file || !&param) return HA_ADMIN_INTERNAL_ERROR;
 
-  myisamchk_init(param);
-  param->thd = thd;
-  param->op_name= "optimize";
-  param->testflag= (check_opt->flags | T_SILENT | T_FORCE_CREATE |
+  myisamchk_init(&param);
+  param.thd = thd;
+  param.op_name= "optimize";
+  param.testflag= (check_opt->flags | T_SILENT | T_FORCE_CREATE |
                    T_REP_BY_SORT | T_STATISTICS | T_SORT_INDEX);
-  param->sort_buffer_length=  THDVAR(thd, sort_buffer_size);
-  if ((error= repair(thd,*param,1)) && param->retry_repair)
+  param.sort_buffer_length=  THDVAR(thd, sort_buffer_size);
+  if ((error= repair(thd,param,1)) && param.retry_repair)
   {
     sql_print_warning("Warning: Optimize table got errno %d on %s.%s, retrying",
-                      my_errno, param->db_name, param->table_name);
-    param->testflag&= ~T_REP_BY_SORT;
-    error= repair(thd,*param,1);
+                      my_errno, param.db_name, param.table_name);
+    param.testflag&= ~T_REP_BY_SORT;
+    error= repair(thd,param,1);
   }
   return error;
 }
@@ -1077,6 +1093,7 @@ int ha_myisam::repair(THD *thd, HA_CHECK &param, bool do_optimize)
   param.thd= thd;
   param.tmpdir= &mysql_tmpdir_list;
   param.out_flag= 0;
+  share->state.dupp_key= MI_MAX_KEY;
   strmov(fixed_name,file->filename);
 
   // Release latches since this can take a long time
@@ -1106,7 +1123,7 @@ int ha_myisam::repair(THD *thd, HA_CHECK &param, bool do_optimize)
 			share->state.key_map);
     ulonglong testflag= param.testflag;
 #ifdef HAVE_MMAP
-    bool remap= test(share->file_map);
+    bool remap= MY_TEST(share->file_map);
     /*
       mi_repair*() functions family use file I/O even if memory
       mapping is available.
@@ -1128,22 +1145,26 @@ int ha_myisam::repair(THD *thd, HA_CHECK &param, bool do_optimize)
         /* TODO: respect myisam_repair_threads variable */
         thd_proc_info(thd, "Parallel repair");
         error = mi_repair_parallel(&param, file, fixed_name,
-                                   test(param.testflag & T_QUICK));
+                                   MY_TEST(param.testflag & T_QUICK));
       }
       else
       {
         thd_proc_info(thd, "Repair by sorting");
+        DEBUG_SYNC(thd, "myisam_before_repair_by_sort");
         error = mi_repair_by_sort(&param, file, fixed_name,
-                                  test(param.testflag & T_QUICK));
-        DEBUG_SYNC(thd, "myisam_after_repair_by_sort");
+                                  MY_TEST(param.testflag & T_QUICK));
       }
+      if (error && file->create_unique_index_by_sort && 
+          share->state.dupp_key != MAX_KEY)
+        print_keydup_error(table, &table->key_info[share->state.dupp_key],
+                           MYF(0));
     }
     else
     {
       thd_proc_info(thd, "Repair with keycache");
       param.testflag &= ~T_REP_BY_SORT;
       error=  mi_repair(&param, file, fixed_name,
-			test(param.testflag & T_QUICK));
+                        MY_TEST(param.testflag & T_QUICK));
     }
     param.testflag= testflag | (param.testflag & T_RETRY_WITHOUT_QUICK);
 #ifdef HAVE_MMAP
@@ -1191,14 +1212,10 @@ int ha_myisam::repair(THD *thd, HA_CHECK &param, bool do_optimize)
     if (file->s->base.auto_key)
       update_auto_increment_key(&param, file, 1);
     if (optimize_done)
-    {
-      mysql_mutex_lock(&share->intern_lock);
       error = update_state_info(&param, file,
 				UPDATE_TIME | UPDATE_OPEN_COUNT |
 				(local_testflag &
 				 T_STATISTICS ? UPDATE_STAT : 0));
-      mysql_mutex_unlock(&share->intern_lock);
-    }
     info(HA_STATUS_NO_LOCK | HA_STATUS_TIME | HA_STATUS_VARIABLE |
 	 HA_STATUS_CONST);
     if (rows != file->state->records && ! (param.testflag & T_VERY_SILENT))
@@ -1257,17 +1274,17 @@ int ha_myisam::assign_to_keycache(THD* thd, HA_CHECK_OPT *check_opt)
   if (error != HA_ADMIN_OK)
   {
     /* Send error to user */
-    HA_CHECK *param= (HA_CHECK*) thd->alloc(sizeof *param);
-    if (!param)
+    HA_CHECK &param= *(HA_CHECK*) thd->alloc(sizeof(param));
+    if (!&param)
       return HA_ADMIN_INTERNAL_ERROR;
 
-    myisamchk_init(param);
-    param->thd= thd;
-    param->op_name=    "assign_to_keycache";
-    param->db_name=    table->s->db.str;
-    param->table_name= table->s->table_name.str;
-    param->testflag= 0;
-    mi_check_print_error(param, errmsg);
+    myisamchk_init(&param);
+    param.thd= thd;
+    param.op_name=    "assign_to_keycache";
+    param.db_name=    table->s->db.str;
+    param.table_name= table->s->table_name.str;
+    param.testflag= 0;
+    mi_check_print_error(&param, errmsg);
   }
   DBUG_RETURN(error);
 }
@@ -1324,16 +1341,16 @@ int ha_myisam::preload_keys(THD* thd, HA_CHECK_OPT *check_opt)
 
  err:
   {
-    HA_CHECK *param= (HA_CHECK*) thd->alloc(sizeof *param);
-    if (!param)
+    HA_CHECK &param= *(HA_CHECK*) thd->alloc(sizeof(param));
+    if (!&param)
       return HA_ADMIN_INTERNAL_ERROR;
-    myisamchk_init(param);
-    param->thd= thd;
-    param->op_name=    "preload_keys";
-    param->db_name=    table->s->db.str;
-    param->table_name= table->s->table_name.str;
-    param->testflag=   0;
-    mi_check_print_error(param, errmsg);
+    myisamchk_init(&param);
+    param.thd= thd;
+    param.op_name=    "preload_keys";
+    param.db_name=    table->s->db.str;
+    param.table_name= table->s->table_name.str;
+    param.testflag=   0;
+    mi_check_print_error(&param, errmsg);
     DBUG_RETURN(error);
   }
 }
@@ -1437,50 +1454,52 @@ int ha_myisam::enable_indexes(uint mode)
   else if (mode == HA_KEY_SWITCH_NONUNIQ_SAVE)
   {
     THD *thd= table->in_use;
-    int was_error= thd->is_error();
-    HA_CHECK *param= (HA_CHECK*) thd->alloc(sizeof *param);
+    HA_CHECK &param= *(HA_CHECK*) thd->alloc(sizeof(param));
     const char *save_proc_info=thd->proc_info;
 
-    if (!param)
+    if (!&param)
       DBUG_RETURN(HA_ADMIN_INTERNAL_ERROR);
 
     thd_proc_info(thd, "Creating index");
-    myisamchk_init(param);
-    param->op_name= "recreating_index";
-    param->testflag= (T_SILENT | T_REP_BY_SORT | T_QUICK |
+    myisamchk_init(&param);
+    param.op_name= "recreating_index";
+    param.testflag= (T_SILENT | T_REP_BY_SORT | T_QUICK |
                      T_CREATE_MISSING_KEYS);
     /*
       Don't lock and unlock table if it's locked.
       Normally table should be locked.  This test is mostly for safety.
     */
     if (likely(file->lock_type != F_UNLCK))
-      param->testflag|= T_NO_LOCKS;
+      param.testflag|= T_NO_LOCKS;
     
-    param->myf_rw&= ~MY_WAIT_IF_FULL;
-    param->sort_buffer_length=  THDVAR(thd, sort_buffer_size);
-    param->stats_method= (enum_handler_stats_method)THDVAR(thd, stats_method);
-    param->tmpdir=&mysql_tmpdir_list;
-    if ((error= (repair(thd,*param,0) != HA_ADMIN_OK)) && param->retry_repair)
+    if (file->create_unique_index_by_sort)
+      param.testflag|= T_CREATE_UNIQUE_BY_SORT;
+
+    param.myf_rw&= ~MY_WAIT_IF_FULL;
+    param.sort_buffer_length=  THDVAR(thd, sort_buffer_size);
+    param.stats_method= (enum_handler_stats_method)THDVAR(thd, stats_method);
+    param.tmpdir=&mysql_tmpdir_list;
+    if ((error= (repair(thd,param,0) != HA_ADMIN_OK)) && param.retry_repair)
     {
       sql_print_warning("Warning: Enabling keys got errno %d on %s.%s, retrying",
-                        my_errno, param->db_name, param->table_name);
+                        my_errno, param.db_name, param.table_name);
       /*
         Repairing by sort failed. Now try standard repair method.
         Still we want to fix only index file. If data file corruption
         was detected (T_RETRY_WITHOUT_QUICK), we shouldn't do much here.
         Let implicit repair do this job.
       */
-      if (!(param->testflag & T_RETRY_WITHOUT_QUICK))
+      if (!(param.testflag & T_RETRY_WITHOUT_QUICK))
       {
-        param->testflag&= ~T_REP_BY_SORT;
-        error= (repair(thd,*param,0) != HA_ADMIN_OK);
+        param.testflag&= ~T_REP_BY_SORT;
+        error= (repair(thd,param,0) != HA_ADMIN_OK);
       }
       /*
         If the standard repair succeeded, clear all error messages which
         might have been set by the first repair. They can still be seen
         with SHOW WARNINGS then.
       */
-      if (! error && ! was_error)
+      if (! error)
         thd->clear_error();
     }
     info(HA_STATUS_CONST);
@@ -1523,20 +1542,21 @@ int ha_myisam::indexes_are_disabled(void)
   activate special bulk-insert optimizations
 
   SYNOPSIS
-    start_bulk_insert(rows)
+    start_bulk_insert(rows, flags)
     rows        Rows to be inserted
                 0 if we don't know
+    flags       Flags to control index creation
 
   NOTICE
     Do not forget to call end_bulk_insert() later!
 */
 
-void ha_myisam::start_bulk_insert(ha_rows rows)
+void ha_myisam::start_bulk_insert(ha_rows rows, uint flags)
 {
   DBUG_ENTER("ha_myisam::start_bulk_insert");
   THD *thd= table->in_use;
-  ulong size= min(thd->variables.read_buff_size,
-                  (ulong) (table->s->avg_row_length*rows));
+  ulong size= MY_MIN(thd->variables.read_buff_size,
+                     (ulong) (table->s->avg_row_length*rows));
   DBUG_PRINT("info",("start_bulk_insert: rows %lu size %lu",
                      (ulong) rows, size));
 
@@ -1547,34 +1567,34 @@ void ha_myisam::start_bulk_insert(ha_rows rows)
   can_enable_indexes= mi_is_all_keys_active(file->s->state.key_map,
                                             file->s->base.keys);
 
-  if (!(specialflag & SPECIAL_SAFE_MODE))
+  /*
+    Only disable old index if the table was empty and we are inserting
+    a lot of rows.
+    Note that in end_bulk_insert() we may truncate the table if
+    enable_indexes() failed, thus it's essential that indexes are
+    disabled ONLY for an empty table.
+  */
+  if (file->state->records == 0 && can_enable_indexes &&
+      (!rows || rows >= MI_MIN_ROWS_TO_DISABLE_INDEXES))
   {
-    /*
-      Only disable old index if the table was empty and we are inserting
-      a lot of rows.
-      Note that in end_bulk_insert() we may truncate the table if
-      enable_indexes() failed, thus it's essential that indexes are
-      disabled ONLY for an empty table.
-    */
-    if (file->state->records == 0 && can_enable_indexes &&
-        (!rows || rows >= MI_MIN_ROWS_TO_DISABLE_INDEXES))
+    if (file->open_flag & HA_OPEN_INTERNAL_TABLE)
     {
-      if (file->open_flag & HA_OPEN_INTERNAL_TABLE)
-      {
-        file->update|= HA_STATE_CHANGED;
-        mi_clear_all_keys_active(file->s->state.key_map);
-      }
-      else
-        mi_disable_non_unique_index(file,rows);
+      file->update|= HA_STATE_CHANGED;
+      mi_clear_all_keys_active(file->s->state.key_map);
     }
     else
+    {
+      my_bool all_keys= MY_TEST(flags & HA_CREATE_UNIQUE_INDEX_BY_SORT);
+      mi_disable_indexes_for_rebuild(file, rows, all_keys);
+    }
+  }
+  else
     if (!file->bulk_insert &&
         (!rows || rows >= MI_MIN_ROWS_TO_USE_BULK_INSERT))
     {
       mi_init_bulk_insert(file, (size_t) thd->variables.bulk_insert_buff_size,
                           rows);
     }
-  }
   DBUG_VOID_RETURN;
 }
 
@@ -1675,8 +1695,6 @@ bool ha_myisam::is_crashed() const
 
 int ha_myisam::update_row(const uchar *old_data, uchar *new_data)
 {
-  if (table->timestamp_field_type & TIMESTAMP_AUTO_SET_ON_UPDATE)
-    table->timestamp_field->set_time();
   return mi_update(file,old_data,new_data);
 }
 
@@ -1854,15 +1872,14 @@ int ha_myisam::info(uint flag)
       number of records in the buffer results in a different number of buffer
       refills and in a different order of records in the result set.
     */
-    stats.mrr_length_per_rec= misam_info.reflength + 8; // 8=max(sizeof(void *))
+    stats.mrr_length_per_rec= misam_info.reflength + 8; // 8=MY_MAX(sizeof(void *))
 
     ref_length= misam_info.reflength;
     share->db_options_in_use= misam_info.options;
     stats.block_size= myisam_block_size;        /* record block size */
 
-    /* Update share */
-    if (share->tmp_table == NO_TMP_TABLE)
-      mysql_mutex_lock(&share->LOCK_ha_data);
+    if (table_share->tmp_table == NO_TMP_TABLE)
+      mysql_mutex_lock(&table_share->LOCK_share);
     share->keys_in_use.set_prefix(share->keys);
     share->keys_in_use.intersect_extended(misam_info.key_map);
     share->keys_for_keyread.intersect(share->keys_in_use);
@@ -1871,29 +1888,22 @@ int ha_myisam::info(uint flag)
       memcpy((char*) table->key_info[0].rec_per_key,
 	     (char*) misam_info.rec_per_key,
              sizeof(table->key_info[0].rec_per_key[0])*share->key_parts);
-    if (share->tmp_table == NO_TMP_TABLE)
-      mysql_mutex_unlock(&share->LOCK_ha_data);
+    if (table_share->tmp_table == NO_TMP_TABLE)
+      mysql_mutex_unlock(&table_share->LOCK_share);
 
    /*
      Set data_file_name and index_file_name to point at the symlink value
      if table is symlinked (Ie;  Real name is not same as generated name)
    */
-    char buf[FN_REFLEN];
     data_file_name= index_file_name= 0;
     fn_format(name_buff, file->filename, "", MI_NAME_DEXT,
               MY_APPEND_EXT | MY_UNPACK_FILENAME);
-    if (my_is_symlink(name_buff))
-    {
-      my_readlink(buf, name_buff, MYF(0));
-      data_file_name= ha_thd()->strdup(buf);
-    }
+    if (strcmp(name_buff, misam_info.data_file_name))
+      data_file_name=misam_info.data_file_name;
     fn_format(name_buff, file->filename, "", MI_NAME_IEXT,
               MY_APPEND_EXT | MY_UNPACK_FILENAME);
-    if (my_is_symlink(name_buff))
-    {
-      my_readlink(buf, name_buff, MYF(0));
-      index_file_name= ha_thd()->strdup(buf);
-    }
+    if (strcmp(name_buff, misam_info.index_file_name))
+      index_file_name=misam_info.index_file_name;
   }
   if (flag & HA_STATUS_ERRKEY)
   {
@@ -1911,8 +1921,6 @@ int ha_myisam::info(uint flag)
 
 int ha_myisam::extra(enum ha_extra_function operation)
 {
-  if ((specialflag & SPECIAL_SAFE_MODE) && operation == HA_EXTRA_KEYREAD)
-    return 0;
   if (operation == HA_EXTRA_MMAP && !opt_myisam_use_mmap)
     return 0;
   return mi_extra(file, operation, 0);
@@ -1930,8 +1938,6 @@ int ha_myisam::reset(void)
 
 int ha_myisam::extra_opt(enum ha_extra_function operation, ulong cache_size)
 {
-  if ((specialflag & SPECIAL_SAFE_MODE) && operation == HA_EXTRA_WRITE_CACHE)
-    return 0;
   return mi_extra(file, operation, (void*) &cache_size);
 }
 
@@ -2022,12 +2028,29 @@ int ha_myisam::create(const char *name, register TABLE *table_arg,
                                (ulonglong) 0);
   create_info.data_file_length= ((ulonglong) share->max_rows *
                                  share->avg_row_length);
-  create_info.data_file_name= ha_create_info->data_file_name;
-  create_info.index_file_name= ha_create_info->index_file_name;
   create_info.language= share->table_charset->number;
 
-  if (ha_create_info->options & HA_LEX_CREATE_TMP_TABLE)
-    create_flags|= HA_CREATE_TMP_TABLE;
+#ifdef HAVE_READLINK
+  if (my_use_symdir)
+  {
+    create_info.data_file_name= ha_create_info->data_file_name;
+    create_info.index_file_name= ha_create_info->index_file_name;
+  }
+  else
+#endif /* HAVE_READLINK */
+  {
+    if (ha_create_info->data_file_name)
+      push_warning_printf(table_arg->in_use, Sql_condition::WARN_LEVEL_WARN,
+                          WARN_OPTION_IGNORED, ER(WARN_OPTION_IGNORED),
+                          "DATA DIRECTORY");
+    if (ha_create_info->index_file_name)
+      push_warning_printf(table_arg->in_use, Sql_condition::WARN_LEVEL_WARN,
+                          WARN_OPTION_IGNORED, ER(WARN_OPTION_IGNORED),
+                          "INDEX DIRECTORY");
+  }
+
+  if (ha_create_info->tmp_table())
+    create_flags|= HA_CREATE_TMP_TABLE | HA_CREATE_DELAY_KEY_WRITE;
   if (ha_create_info->options & HA_CREATE_KEEP_FILES)
     create_flags|= HA_CREATE_KEEP_FILES;
   if (options & HA_OPTION_PACK_RECORD)
@@ -2225,6 +2248,7 @@ static int myisam_init(void *p)
   myisam_hton->create= myisam_create_handler;
   myisam_hton->panic= myisam_panic;
   myisam_hton->flags= HTON_CAN_RECREATE | HTON_SUPPORT_LOG_TABLES;
+  myisam_hton->tablefile_extensions= ha_myisam_exts;
   mi_killed= mi_killed_in_mariadb;
 
   return 0;
@@ -2262,7 +2286,7 @@ int ha_myisam::multi_range_read_next(range_id_t *range_info)
 ha_rows ha_myisam::multi_range_read_info_const(uint keyno, RANGE_SEQ_IF *seq,
                                                void *seq_init_param, 
                                                uint n_ranges, uint *bufsz,
-                                               uint *flags, COST_VECT *cost)
+                                               uint *flags, Cost_estimate *cost)
 {
   /*
     This call is here because there is no location where this->table would
@@ -2276,7 +2300,7 @@ ha_rows ha_myisam::multi_range_read_info_const(uint keyno, RANGE_SEQ_IF *seq,
 
 ha_rows ha_myisam::multi_range_read_info(uint keyno, uint n_ranges, uint keys,
                                          uint key_parts, uint *bufsz, 
-                                         uint *flags, COST_VECT *cost)
+                                         uint *flags, Cost_estimate *cost)
 {
   ds_mrr.init(this, table);
   return ds_mrr.dsmrr_info(keyno, n_ranges, keys, key_parts, bufsz, flags, cost);
@@ -2306,7 +2330,7 @@ Item *ha_myisam::idx_cond_push(uint keyno_arg, Item* idx_cond_arg)
   */
   const KEY *key= &table_share->key_info[keyno_arg];
 
-  for (uint k= 0; k < key->key_parts; ++k)
+  for (uint k= 0; k < key->user_defined_key_parts; ++k)
   {
     const KEY_PART_INFO *key_part= &key->key_part[k];
     if (key_part->key_part_flag & HA_BLOB_PART)
@@ -2350,7 +2374,7 @@ maria_declare_plugin(myisam)
   &myisam_storage_engine,
   "MyISAM",
   "MySQL AB",
-  "Non-transactional engine with good performance and small data footprint",
+  "MyISAM storage engine",
   PLUGIN_LICENSE_GPL,
   myisam_init, /* Plugin Init */
   NULL, /* Plugin Deinit */
@@ -2381,7 +2405,7 @@ maria_declare_plugin_end;
 
   @return The error code. The engine_data and engine_callback will be set to 0.
     @retval TRUE Success
-    @retval FALSE An error occured
+    @retval FALSE An error occurred
 */
 
 my_bool ha_myisam::register_query_cache_table(THD *thd, char *table_name,
