@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /* write whats in isam.log */
 
@@ -91,7 +91,7 @@ int main(int argc, char **argv)
   log_filename=myisam_log_filename;
   get_options(&argc,&argv);
   /* Number of MyISAM files we can have open at one time */
-  max_files= (my_set_max_open_files(min(max_files,8))-6)/2;
+  max_files= (my_set_max_open_files(MY_MIN(max_files,8))-6)/2;
   if (update)
     printf("Trying to %s MyISAM files according to log '%s'\n",
 	   (recover ? "recover" : "update"),log_filename);
@@ -266,7 +266,7 @@ static void get_options(register int *argc, register char ***argv)
 	puts("         -u \"update\"         -v \"verbose\"   -w \"write file\"");
 	puts("         -D \"myisam compiled with DBUG\"   -P \"processes\"");
 	puts("\nOne can give a second and a third '-v' for more verbose.");
-	puts("Normaly one does a update (-u).");
+	puts("Normally one does a update (-u).");
 	puts("If a recover is done all writes and all possibly updates and deletes is done\nand errors are only counted.");
 	puts("If one gives table names as arguments only these tables will be updated\n");
 	help=1;
@@ -329,10 +329,11 @@ static int examine_log(char * file_name, char **table_names)
 
   init_io_cache(&cache,file,0,READ_CACHE,start_offset,0,MYF(0));
   bzero((uchar*) com_count,sizeof(com_count));
-  init_tree(&tree,0,0,sizeof(file_info),(qsort_cmp2) file_info_compare,1,
-	    (tree_element_free) file_info_free, NULL);
+  init_tree(&tree,0,0,sizeof(file_info),(qsort_cmp2) file_info_compare,
+	    (tree_element_free) file_info_free, NULL,
+            MYF(MY_TREE_WITH_DELETE));
   (void) init_key_cache(dflt_key_cache,KEY_CACHE_BLOCK_SIZE,KEY_CACHE_SIZE,
-                      0, 0, 0);
+                        0, 0, 0, 0);
 
   files_open=0; access_time=0;
   while (access_time++ != number_of_commands &&
@@ -415,7 +416,7 @@ static int examine_log(char * file_name, char **table_names)
 		     left_root_right);
       file_info.id=open_param.max_id+1;
       /*
-       * In the line below +10 is added to accommodate '<' and '>' chars
+       * In the line below +10 is added to accomodate '<' and '>' chars
        * plus '\0' at the end, so that there is place for 7 digits.
        * It is  improbable that same table can have that many entries in 
        * the table cache.
